# coding: utf-8


class VertexAttribute(object):
    __slots__=[
            'nx', 'ny', 'nz', # normal
            'u', 'v', # uv
            ]
    def __init__(self, nx, ny, nz, u, v):
        self.nx=nx
        self.ny=ny
        self.nz=nz
        self.u=u
        self.v=v

    def __str__(self):
        return "<vkey: %f, %f, %f, %f, %f>" % (
                self.nx, self.ny, self.nz, self.u, self.v)

    def __hash__(self):
        return int(100*(self.nx + self.ny + self.nz + self.u + self.v))

    def __eq__(self, rhs):
        return self.nx==rhs.nx and self.ny==rhs.ny and self.nz==rhs.nz and self.u==rhs.u and self.v==rhs.v


class VertexKey(object):
    __slots__=[
            'obj_index', 'index',
            ]

    def __init__(self, obj_index, index):
        self.obj_index=obj_index
        self.index=index

    def __str__(self):
        return "<vkey: %d, %d>" % (self.obj_index, self.index)

    def __hash__(self):
        return self.index*100+self.obj_index

    def __eq__(self, rhs):
        return self.obj_index==rhs.obj_index and self.index==rhs.index


class VertexArray(object):
    """
    頂点配列
    """
    __slots__=[
            'indexArrays',
            'positions',
            'attributes', # normal and uv
            'b0', 'b1', 'weight',
            'vertexMap',
            'objectMap',
            ]
    def __init__(self):
        # indexArrays split with each material
        self.indexArrays={}

        self.positions=[]
        self.attributes=[]
        self.b0=[]
        self.b1=[]
        self.weight=[]

        self.vertexMap={}
        self.objectMap={}

    def __str__(self):
        return "<VertexArray %d positions, %d indexArrays>" % (
                len(self.positions), len(self.indexArrays))

    def zip(self):
        return zip(
                self.positions, self.attributes,
                self.b0, self.b1, self.weight)

    def each(self):
        keys=[key for key in self.indexArrays.keys()]
        keys.sort()
        for key in keys:
            yield(key, self.indexArrays[key])

    def __addOrGetIndex(self, obj_index, base_index, pos, normal, uv, b0, b1, weight0):
        key=VertexKey(obj_index, base_index)
        attribute=VertexAttribute( 
                normal[0], normal[1], normal[2],
                uv[0], uv[1])
        if key in self.vertexMap:
            if attribute in self.vertexMap[key]:
                return self.vertexMap[key][attribute]
            else:
                return self.__addVertex(self.vertexMap[key],
                        pos, attribute, b0, b1, weight0)
        else:
            vertexMapKey={}
            self.vertexMap[key]=vertexMapKey
            return self.__addVertex(vertexMapKey,
                    pos, attribute, b0, b1, weight0)

    def __addVertex(self, vertexMapKey, pos, attribute, b0, b1, weight0):
        index=len(self.positions)
        vertexMapKey[attribute]=index
        # position
        self.positions.append((pos.x, pos.y, pos.z))
        # unique attribute
        self.attributes.append(attribute)
        # shared attribute
        self.b0.append(b0)
        self.b1.append(b1)
        self.weight.append(weight0)
        assert(index<=65535)
        return index
            
    def getMappedIndex(self, obj_name, base_index):
        return self.vertexMap[VertexKey(self.objectMap[obj_name], base_index)]

    def addTriangle(self,
            object_name, material,
            base_index0, base_index1, base_index2,
            pos0, pos1, pos2,
            n0, n1, n2,
            uv0, uv1, uv2,
            b0_0, b0_1, b0_2,
            b1_0, b1_1, b1_2,
            weight0, weight1, weight2
            ):
        if object_name in self.objectMap:
            obj_index=self.objectMap[object_name]
        else:
            obj_index=len(self.objectMap)
            self.objectMap[object_name]=obj_index
        index0=self.__addOrGetIndex(obj_index, base_index0, pos0, n0, uv0, b0_0, b1_0, weight0)
        index1=self.__addOrGetIndex(obj_index, base_index1, pos1, n1, uv1, b0_1, b1_1, weight1)
        index2=self.__addOrGetIndex(obj_index, base_index2, pos2, n2, uv2, b0_2, b1_2, weight2)

        if not material in self.indexArrays:
            self.indexArrays[material]=[]
        self.indexArrays[material]+=[index0, index1, index2]




#!/usr/bin/env python
# coding: utf8
"""
日本語名との変換マップ
"""
import sys

"""
ボーン名変換
"""
boneMap=[
("center", "センター", 1),
("upper body", "上半身"),
("neck", "首"),
("head", "頭"),
("eye_L", "左目", 5),
("eye_R", "右目", 5),
("necktie1", "ﾈｸﾀｲ１"),
("necktie2", "ﾈｸﾀｲ２"),
("necktie3", "ﾈｸﾀｲ３"),
("lower body", "下半身"),
("waist accessory", "腰飾り"),
("hair1_L", "左髪１"),
("hair2_L", "左髪２"),
("hair3_L", "左髪３"),
("hair4_L", "左髪４"),
("hair5_L", "左髪５"),
("hair6_L", "左髪６"),
("shoulder_L", "左肩"),
("arm_L", "左腕"),
("arm twist_L", "左腕捩", 8),
("elbow_L", "左ひじ"),
("wrist twist_L", "左手捩", 8),
("wrist_L", "左手首"),
("sleeve_L", "左袖", 1),
("thumb1_L", "左親指１"),
("thumb2_L", "左親指２"),
("fore1_L", "左人指１"),
("fore2_L", "左人指２"),
("fore3_L", "左人指３"),
("middle1_L", "左中指１"),
("middle2_L", "左中指２"),
("middle3_L", "左中指３"),
("third1_L", "左薬指１"),
("third2_L", "左薬指２"),
("third3_L", "左薬指３"),
("little1_L", "左小指１"),
("little2_L", "左小指２"),
("little3_L", "左小指３"),
("front skirt_L", "左ｽｶｰﾄ前"),
("back skirt_L", "左ｽｶｰﾄ後"),
("leg_L", "左足"),
("knee_L", "左ひざ"),
("ankle_L", "左足首"),
("hair1_R", "右髪１"),
("hair2_R", "右髪２"),
("hair3_R", "右髪３"),
("hair4_R", "右髪４"),
("hair5_R", "右髪５"),
("hair6_R", "右髪６"),
("shoulder_R", "右肩"),
("arm_R", "右腕"),
("arm twist_R", "右腕捩", 8),
("elbow_R", "右ひじ"),
("wrist twist_R", "右手捩", 8),
("wrist_R", "右手首"),
("sleeve_R", "右袖", 1),
("thumb1_R", "右親指１"),
("thumb2_R", "右親指２"),
("fore1_R", "右人指１"),
("fore2_R", "右人指２"),
("fore3_R", "右人指３"),
("middle1_R", "右中指１"),
("middle2_R", "右中指２"),
("middle3_R", "右中指３"),
("third1_R", "右薬指１"),
("third2_R", "右薬指２"),
("third3_R", "右薬指３"),
("little1_R", "右小指１"),
("little2_R", "右小指２"),
("little3_R", "右小指３"),
("front skirt_R", "右ｽｶｰﾄ前"),
("back skirt_R", "右ｽｶｰﾄ後"),
("leg_R", "右足"),
("knee_R", "右ひざ"),
("ankle_R", "右足首"),
("eyes", "両目"),
("front hair1", "前髪１"),
("front hair2", "前髪２"),
("front hair3", "前髪３"),
("eyelight_L", "左目光"),
("eyelight_R", "右目光"),
("necktie3_t", "ﾈｸﾀｲ４"),
("hair6_L_t", "左髪７"),
("hair6_R_t", "右髪７"),
("ankle_L_t", "左つま先"),
("ankle_R_t", "右つま先"),
("necktie IK", "ﾈｸﾀｲＩＫ"),
("hair IK_L", "左髪ＩＫ"),
("hair IK_R", "右髪ＩＫ"),
("leg IK_L", "左足ＩＫ"),
("leg IK_R", "右足ＩＫ"),
("toe IK_L", "左つま先ＩＫ"),
("toe IK_R", "右つま先ＩＫ"),

("lower body_t", "下半身先"),
("head_t", "頭先"),
("eye_L_t", "左目先"),
("eye_R_t", "右目先"),
("waist accessory_t", "腰飾り先"),

("sleeve_L_t", "左袖先"),
("wrist_L_t", "左手先"),
("thumb2_L_t", "左親指先"),
("fore3_L_t", "左人差指先"),
("middle3_L_t", "左中指先"),
("third3_L_t", "左薬指先"),
("little3_L_t", "左小指先"),
("front skirt_L_t", "左スカート前先"),
("back skirt_L_t", "左スカート後先"),

("sleeve_R_t", "右袖先"),
("wrist_R_t", "右手先"),
("thumb2_R_t", "右親指先"),
("fore3_R_t", "右人差指先"),
("middle3_R_t", "右中指先"),
("third3_R_t", "右薬指先"),
("little3_R_t", "右小指先"),
("front skirt_R_t", "右スカート前先"),
("back skirt_R_t", "右スカート後先"),

("center_t", "センター先"),
("eyes_t", "両目先"),
("necktie IK_t", "ﾈｸﾀｲＩＫ先"),
("hair IK_L_t", "左髪ＩＫ先"),
("hair IK_R_t", "右髪ＩＫ先"),
("leg IK_L_t", "左足ＩＫ先"),
("leg IK_R_t", "右足ＩＫ先"),
("toe IK_L_t", "左つま先ＩＫ先"),
("toe IK_R_t", "右つま先ＩＫ先"),
("front hair1_t", "前髪１先"),
("front hair2_t", "前髪２先"),
("front hair3_t", "前髪３先"),
("eyelight_L_t", "左目光先"),
("eyelight_R_t", "右目光先"),
("arm twist_L_t", "左腕捩先"),
("wrist twist_L_t", "左手捩先"),
("arm twist_R_t", "右腕捩先"),
("wrist twist_R_t", "右手捩先"),
("arm twist1_L", "左腕捩1", 9),
("arm twist2_L", "左腕捩2", 9),
("arm twist3_L", "左腕捩3", 9),
("arm twist1_R", "右腕捩1", 9),
("arm twist2_R", "右腕捩2", 9),
("arm twist3_R", "右腕捩3", 9),
#
("arm twist1_L_t", "左腕捩1先"),
("arm twist2_L_t", "左腕捩2先"),
("arm twist3_L_t", "左腕捩3先"),
("arm twist1_R_t", "右腕捩1先"),
("arm twist2_R_t", "右腕捩2先"),
("arm twist3_R_t", "右腕捩3先"),

# 追加ボーン
("root", "全ての親"),
("root_t", "全ての親先"),
("group", "グループ"),
("group_t", "グループ先"),
("front_shirt_L", "左シャツ前"),
("front_shirt_R", "右シャツ前"),
("back_shirt_L", "左シャツ後"),
("back_shirt_R", "右シャツ後"),
]
def getEnglishBoneName(name):
    for v in boneMap:
        if v[1]==name:
            return v[0]

def getIndexByEnglish(name):
    for i, v in enumerate(boneMap):
        if v[0]==name:
            return i

def getUnicodeBoneName(name):
    for v in boneMap:
        if v[0]==name:
            return v

"""
モーフ名変換
"""
skinMap=[
("base", "base", 0),
("serious", "真面目", 1),
("sadness", "困る", 1),
("cheerful", "にこり", 1),
("anger", "怒り", 1),
("go up", "上", 1),
("go down", "下", 1),
("blink", "まばたき", 2),
("smile", "笑い", 2),
("wink", "ウィンク", 2),
("wink2", "ウィンク２", 2),
("wink_R", "ウィンク右", 2),
("wink2_R", "ｳｨﾝｸ２右", 2),
("close><", "はぅ", 2),
("calm", "なごみ", 2),
("surprise", "びっくり", 2),
("doubt", "じと目", 2),
("confuse", "なぬ！", 2),
("pupil", "瞳小", 4),
("a", "あ", 3),
("i", "い", 3),
("u", "う", 3),
("o", "お", 3),
("triangle", "▲", 3),
("regret", "∧", 3),
("omega", "ω", 3),
("omegabox", "ω□", 3),
("fool", "はんっ！", 3),
("tongue", "ぺろっ", 4),
("e-", "えー", 3),
("grin", "にやり", 3),
]
def getEnglishSkinName(name):
    for v in skinMap:
        if v[1]==name:
            return v[0]

def getUnicodeSkinName(name):
    for v in skinMap:
        if v[0]==name:
            return v

"""
ボーングループ名変換
"""
boneGroupMap=[
        ("IK", "ＩＫ"),
        ("Body[u]", "体(上)"),
        ("Hair", "髪"),
        ("Arms", "腕"),
        ("Fingers", "指"),
        ("Body[l]", "体(下)"),
        ("Legs", "足"),
        ]
def getEnglishBoneGroupName(name):
    for v in boneGroupMap:
        if v[1]==name:
            return v[0]

def getUnicodeBoneGroupName(name):
    for v in boneGroupMap:
        if v[0]==name:
            return v[1]


###############################################################################
# blender2.4 str to unicode
###############################################################################
if sys.version_info[0]<3:
    print('convert boneMap and skinMap to unicode...')
    # python2.x
    # unicodeに変換
    for i, l in enumerate(boneMap):
        replace=[]
        for j, m in enumerate(l):
            if j==1:
                replace.append(m.decode('utf-8'))
            else:
                replace.append(m)
        boneMap[i]=replace

    for i, l in enumerate(skinMap):
        replace=[]
        for j, m in enumerate(l):
            if j==1:
                replace.append(m.decode('utf-8'))
            else:
                replace.append(m)
        skinMap[i]=replace
    print('done')        




# coding: utf-8

import io
from . import bl
from . import exporter
from .pymeshio import pmx
from .pymeshio import common
from .pymeshio.pmx import writer


def near(x, y, EPSILON=1e-5):
    d=x-y
    return d>=-EPSILON and d<=EPSILON


def create_pmx(ex):
    model=pmx.Model()

    o=ex.root.o
    model.english_name=o.name
    model.name=o[bl.MMD_MB_NAME] if bl.MMD_MB_NAME in o else 'Blenderエクスポート'
    model.comment=o[bl.MMD_MB_COMMENT] if bl.MMD_MB_COMMENT in o else 'Blnderエクスポート\n'
    model.english_comment=o[bl.MMD_COMMENT] if bl.MMD_COMMENT in o else 'blender export\n'

    def get_deform(b0, b1, weight):
        if b0==-1:
            return pmx.Bdef1(b1, weight)
        elif b1==-1:
            return pmx.Bdef1(b0, weight)
        else:
            return pmx.Bdef2(b0, b1, weight)

    model.vertices=[pmx.Vertex(
        # convert right-handed z-up to left-handed y-up
        common.Vector3(pos[0], pos[2], pos[1]), 
        # convert right-handed z-up to left-handed y-up
        common.Vector3(attribute.nx, attribute.nz, attribute.ny),
        # reverse vertical
        common.Vector2(attribute.u, 1.0-attribute.v),
        get_deform(ex.skeleton.indexByName(b0), ex.skeleton.indexByName(b1), weight),
        # edge flag, 0: enable edge, 1: not edge
        1.0 
        )
        for pos, attribute, b0, b1, weight in ex.oneSkinMesh.vertexArray.zip()]

    '''
    # IK
    for ik in self.skeleton.ik_list:
        solver=pmd.IK()
        solver.index=self.skeleton.getIndex(ik.target)
        solver.target=self.skeleton.getIndex(ik.effector)
        solver.length=ik.length
        b=self.skeleton.bones[ik.effector.parent_index]
        for i in range(solver.length):
            solver.children.append(self.skeleton.getIndex(b))
            b=self.skeleton.bones[b.parent_index]
        solver.iterations=ik.iterations
        solver.weight=ik.weight
        model.ik_list.append(solver)
    '''
    def create_bone(b):
        return pmx.Bone(
            name=b.name,
            english_name=b.name,
            # convert right-handed z-up to left-handed y-up
            position=common.Vector3(
                b.pos[0] if not near(b.pos[0], 0) else 0,
                b.pos[2] if not near(b.pos[2], 0) else 0,
                b.pos[1] if not near(b.pos[1], 0) else 0
                ),
            parent_index=b.parent_index,
            layer=0,
            flag=0,
            tail_position=None,
            tail_index=b.tail_index,
            effect_index=-1,
            effect_factor=0.0,
            fixed_axis=None,
            local_x_vector=None,
            local_z_vector=None,
            external_key=-1,
            ik=None
                )
    model.bones=[create_bone(b)
            for b in ex.skeleton.bones]

    textures=set()
    def get_texture_name(texture):
        pos=texture.replace("\\", "/").rfind("/")
        if pos==-1:
            return texture
        else:
            return texture[pos+1:]
    for m in ex.oneSkinMesh.vertexArray.indexArrays.keys():
        for path in bl.material.eachEnalbeTexturePath(bl.material.get(m)):
            textures.add(get_texture_name(path))
    model.textures=list(textures)

    # 面とマテリアル
    vertexCount=ex.oneSkinMesh.getVertexCount()
    for material_name, indices in ex.oneSkinMesh.vertexArray.each():
        #print('material:', material_name)
        try:
            m=bl.material.get(material_name)
        except KeyError as e:
            m=DefaultMatrial()
        # マテリアル
        model.materials.append(pmx.Material(
                name=m.name,
                english_name='',
                diffuse_color=common.RGB(m.diffuse_color[0], m.diffuse_color[1], m.diffuse_color[2]),
                alpha=m.alpha,
                specular_factor=0 if m.specular_toon_size<1e-5 else m.specular_hardness*10,
                specular_color=common.RGB(m.specular_color[0], m.specular_color[1], m.specular_color[2]),
                ambient_color=common.RGB(m.mirror_color[0], m.mirror_color[1], m.mirror_color[2]),
                flag=1 if m.subsurface_scattering.use else 0,
                edge_color=common.RGBA(0, 0, 0, 1),
                edge_size=1.0,
                texture_index=0,
                sphere_texture_index=0,
                sphere_mode=0,
                toon_sharing_flag=0,
                toon_texture_index=0,
                comment='',
                vertex_count=len(indices)
                ))
        # 面
        for i in indices:
            assert(i<vertexCount)
        for i in range(0, len(indices), 3):
            # reverse triangle
            model.indices.append(indices[i+2])
            model.indices.append(indices[i+1])
            model.indices.append(indices[i])

    # 表情
    for i, m in enumerate(ex.oneSkinMesh.morphList):
        morph=pmx.Morph(
                name=m.name,
                english_name='',
                panel=0,
                morph_type=1,
                )
        morph.offsets=[pmx.VertexMorphOffset(
            index, 
            common.Vector3(offset[0], offset[2], offset[1])
            )
            for index, offset in m.offsets]
        model.morphs.append(morph)


    # ボーングループ
    model.display_slots=[pmx.DisplaySlot(
        name=name,
        english_name='',
        special_flag=0,
        )
        for name, members in ex.skeleton.bone_groups]

    # rigid body
    boneNameMap={}
    for i, b in enumerate(ex.skeleton.bones):
        boneNameMap[b.name]=i
    rigidNameMap={}
    for i, obj in enumerate(ex.oneSkinMesh.rigidbodies):
        name=obj[bl.RIGID_NAME] if bl.RIGID_NAME in obj else obj.name
        print(name)
        rigidNameMap[name]=i
        boneIndex=boneNameMap[obj[bl.RIGID_BONE_NAME]]
        if boneIndex==0:
            boneIndex=-1
            bone=ex.skeleton.bones[0]
        else:
            bone=ex.skeleton.bones[boneIndex]
        if obj[bl.RIGID_SHAPE_TYPE]==0:
            shape_type=0
            shape_size=common.Vector3(obj.scale[0], 0, 0)
        elif obj[bl.RIGID_SHAPE_TYPE]==1:
            shape_type=1
            shape_size=common.Vector3(obj.scale[0], obj.scale[1], obj.scale[2])
        elif obj[bl.RIGID_SHAPE_TYPE]==2:
            shape_type=2
            shape_size=common.Vector3(obj.scale[0], obj.scale[2], 0)
        rigidBody=pmx.RigidBody(
                name=name, 
                english_name='',
                collision_group=obj[bl.RIGID_GROUP],
                no_collision_group=obj[bl.RIGID_INTERSECTION_GROUP],
                bone_index=boneIndex,
                shape_position=common.Vector3(
                    obj.location.x-bone.pos[0],
                    obj.location.z-bone.pos[2],
                    obj.location.y-bone.pos[1]),
                shape_rotation=common.Vector3(
                    -obj.rotation_euler[0],
                    -obj.rotation_euler[2],
                    -obj.rotation_euler[1]),
                shape_type=shape_type,
                shape_size=shape_size,
                mass=obj[bl.RIGID_WEIGHT],
                linear_damping=obj[bl.RIGID_LINEAR_DAMPING],
                angular_damping=obj[bl.RIGID_ANGULAR_DAMPING],
                restitution=obj[bl.RIGID_RESTITUTION],
                friction=obj[bl.RIGID_FRICTION],
                mode=obj[bl.RIGID_PROCESS_TYPE]
                )
        model.rigidbodies.append(rigidBody)

    # joint
    model.joints=[pmx.Joint(
        name=obj[bl.CONSTRAINT_NAME],
        english_name='',
        joint_type=0,
        rigidbody_index_a=rigidNameMap[obj[bl.CONSTRAINT_A]],
        rigidbody_index_b=rigidNameMap[obj[bl.CONSTRAINT_B]],
        position=common.Vector3(
            obj.location[0], 
            obj.location[2], 
            obj.location[1]),
        rotation=common.Vector3(
            -obj.rotation_euler[0], 
            -obj.rotation_euler[2], 
            -obj.rotation_euler[1]),
        translation_limit_min=common.Vector3(
            obj[bl.CONSTRAINT_POS_MIN][0],
            obj[bl.CONSTRAINT_POS_MIN][1],
            obj[bl.CONSTRAINT_POS_MIN][2]
            ),
        translation_limit_max=common.Vector3(
            obj[bl.CONSTRAINT_POS_MAX][0],
            obj[bl.CONSTRAINT_POS_MAX][1],
            obj[bl.CONSTRAINT_POS_MAX][2]
            ),
        rotation_limit_min=common.Vector3(
            obj[bl.CONSTRAINT_ROT_MIN][0],
            obj[bl.CONSTRAINT_ROT_MIN][1],
            obj[bl.CONSTRAINT_ROT_MIN][2]),
        rotation_limit_max=common.Vector3(
            obj[bl.CONSTRAINT_ROT_MAX][0],
            obj[bl.CONSTRAINT_ROT_MAX][1],
            obj[bl.CONSTRAINT_ROT_MAX][2]),
        spring_constant_translation=common.Vector3(
            obj[bl.CONSTRAINT_SPRING_POS][0],
            obj[bl.CONSTRAINT_SPRING_POS][1],
            obj[bl.CONSTRAINT_SPRING_POS][2]),
        spring_constant_rotation=common.Vector3(
            obj[bl.CONSTRAINT_SPRING_ROT][0],
            obj[bl.CONSTRAINT_SPRING_ROT][1],
            obj[bl.CONSTRAINT_SPRING_ROT][2])
        )
        for obj in ex.oneSkinMesh.constraints]

    return model


def _execute(filepath):
    active=bl.object.getActive()
    if not active:
        print("abort. no active object.")
        return

    ex=exporter.Exporter()
    ex.setup()

    model=create_pmx(ex)
    bl.object.activate(active)
    with io.open(filepath, 'wb') as f:
        writer.write(f, model)
    return {'FINISHED'}




#!/usr/bin/env python
# coding: utf-8

import time
import os
import pymeshio.pmd.reader
import opengl.material
import opengl.texture
import opengl.vertexarray


def build(path):
    # load scenee
    t=time.time()
    model=pymeshio.pmd.reader.read_from_file(path)
    if not model:
        return
    print(time.time()-t, "sec")
    # build
    basedir=os.path.dirname(path)
    indexedVertexArray=opengl.vertexarray.IndexedVertexArray()
    for v in model.vertices:
        # left-handed y-up to right-handed y-up                
        indexedVertexArray.addVertex(
                (v.pos[0], v.pos[1], -v.pos[2], 1), 
                (v.normal[0], v.normal[1], -v.normal[2]), 
                (v.uv[0], v.uv[1]), 
                (1, 1, 1, 1),
                v.bone0, v.bone1, v.weight0)
    
    # material
    textureMap={}
    faceIndex=0
    def indices():
        for i in model.indices:
            yield i
    indexGen=indices()
    for i, m in enumerate(model.materials):
        material=opengl.material.MQOMaterial()
        material.vcol=True
        material.rgba=(
                m.diffuse_color[0], 
                m.diffuse_color[1], 
                m.diffuse_color[2], 
                m.alpha)
        texturefile=m.texture_file.decode('cp932')
        texturepath=os.path.join(basedir, texturefile)
        if os.path.isfile(texturepath):
            if not texturepath in textureMap:
                texture=opengl.texture.Texture(texturepath)
                textureMap[texturepath]=texture
            material.texture=textureMap[texturepath]
        indices=indexedVertexArray.addMaterial(material)
        indices+=[next(indexGen) for n in range(m.vertex_count)]
    indexedVertexArray.optimize()
    return indexedVertexArray




# coding: utf-8
"""
pmx reader
"""
import io
from .. import common
from .. import pmx


class Reader(common.BinaryReader):
    """pmx reader
    """
    def __init__(self, ios,
            text_encoding,
            extended_uv,
            vertex_index_size,
            texture_index_size,
            material_index_size,
            bone_index_size,
            morph_index_size,
            rigidbody_index_size
            ):
        super(Reader, self).__init__(ios)
        self.read_text=self.get_read_text(text_encoding)
        if extended_uv>0:
            raise common.ParseException(
                    "extended uv is not supported", extended_uv)
        self.read_vertex_index=lambda : self.read_int(vertex_index_size)
        self.read_texture_index=lambda : self.read_int(texture_index_size)
        self.read_material_index=lambda : self.read_int(material_index_size)
        self.read_bone_index=lambda : self.read_int(bone_index_size)
        self.read_morph_index=lambda : self.read_int(morph_index_size)
        self.read_rigidbody_index=lambda : self.read_int(rigidbody_index_size)

    def __str__(self):
        return '<pmx.Reader>'

    def get_read_text(self, text_encoding):
        if text_encoding==0:
            def read_text():
                size=self.read_int(4)
                return self.unpack("{0}s".format(size), size).decode("UTF16")
            return read_text
        elif text_encoding==1:
            def read_text():
                size=self.read_int(4)
                return self.unpack("{0}s".format(size), size).decode("UTF8")
            return read_text
        else:
            print("unknown text encoding", text_encoding)

    def read_vertex(self):
        return pmx.Vertex(
                self.read_vector3(), # pos
                self.read_vector3(), # normal
                self.read_vector2(), # uv
                self.read_deform(), # deform(bone weight)
                self.read_float() # edge factor
                )

    def read_deform(self):
        deform_type=self.read_int(1)
        if deform_type==0:
            return pmx.Bdef1(self.read_bone_index())
        elif deform_type==1:
            return pmx.Bdef2(
                    self.read_bone_index(),
                    self.read_bone_index(),
                    self.read_float()
                    )
        elif deform_type==2:
            return pmx.Bdef4(
                    self.read_bone_index(),
                    self.read_bone_index(),
                    self.read_bone_index(),
                    self.read_bone_index(),
                    self.read_float(),
                    self.read_float(),
                    self.read_float(),
                    self.read_float()
                    )
        elif deform_type==3:
            return pmx.Sdef(
                    self.read_bone_index(),
                    self.read_bone_index(),
                    self.read_float(),
                    self.read_vector3(),
                    self.read_vector3(),
                    self.read_vector3()
                    )
        else:
            raise common.ParseException(
                    "unknown deform type: {0}".format(deform_type))

    def read_material(self):
        material=pmx.Material(
                name=self.read_text(),
                english_name=self.read_text(),
                diffuse_color=self.read_rgb(),
                alpha=self.read_float(),
                specular_color=self.read_rgb(),
                specular_factor=self.read_float(),
                ambient_color=self.read_rgb(),
                flag=self.read_int(1),
                edge_color=self.read_rgba(),
                edge_size=self.read_float(),
                texture_index=self.read_texture_index(),
                sphere_texture_index=self.read_texture_index(),
                sphere_mode=self.read_int(1),
                toon_sharing_flag=self.read_int(1),
                )
        if material.toon_sharing_flag==0:
            material.toon_texture_index=self.read_texture_index()
        elif material.toon_sharing_flag==1:
            material.toon_texture_index=self.read_int(1)
        else:
            raise common.ParseException(
                    "unknown toon_sharing_flag {0}".format(
                        material.toon_sharing_flag))
        material.comment=self.read_text()
        material.vertex_count=self.read_int(4)
        return material

    def read_bone(self):
        bone=pmx.Bone(
                name=self.read_text(),
                english_name=self.read_text(),
                position=self.read_vector3(),
                parent_index=self.read_bone_index(),
                layer=self.read_int(4),
                flag=self.read_int(2)                
                )
        if bone.getConnectionFlag()==0:
            bone.tail_position=self.read_vector3()
        elif bone.getConnectionFlag()==1:
            bone.tail_index=self.read_bone_index()
        else:
            raise common.ParseException(
                    "unknown bone conenction flag: {0}".format(
                bone.getConnectionFlag()))

        if bone.getRotationFlag()==1 or bone.getTranslationFlag()==1:
            bone.effect_index=self.read_bone_index()
            bone.effect_factor=self.read_float()

        if bone.getFixedAxisFlag()==1:
            bone.fixed_axis=self.read_vector3()

        if bone.getLocalCoordinateFlag()==1:
            bone.local_x_vector=self.read_vector3()
            bone.local_z_vector=self.read_vector3()

        if bone.getExternalParentDeformFlag()==1:
            bone.external_key=self.read_int(4)

        if bone.getIkFlag()==1:
            bone.ik=self.read_ik()

        return bone

    def read_ik(self):
        ik=pmx.Ik(
                target_index=self.read_bone_index(),
                loop=self.read_int(4),
                limit_radian=self.read_float())
        link_size=self.read_int(4)
        ik.link=[self.read_ik_link() 
                for _ in range(link_size)]
        return ik

    def read_ik_link(self):
        link=pmx.IkLink(
                self.read_bone_index(),
                self.read_int(1))
        if link.limit_angle==0:
            pass
        elif link.limit_angle==1:
            link.limit_min=self.read_vector3()
            link.limit_max=self.read_vector3()
        else:
            raise common.ParseException(
                    "invalid ik link limit_angle: {0}".format(
                link.limit_angle))
        return link

    def read_morgh(self):
        name=self.read_text()
        english_name=self.read_text()
        panel=self.read_int(1)
        morph_type=self.read_int(1)
        offset_size=self.read_int(4)
        morph=pmx.Morph(name, english_name, 
                panel, morph_type)
        if morph_type==0:
            # todo
            raise common.ParseException(
                    "not implemented GroupMorph")
        elif morph_type==1:
            morph.offsets=[self.read_vertex_position_morph_offset() 
                    for _ in range(offset_size)]
        elif morph_type==2:
            # todo
            raise common.ParseException(
                    "not implemented BoneMorph")
        elif morph_type==3:
            # todo
            raise common.ParseException(
                    "not implemented UvMorph")
        elif morph_type==4:
            # todo
            raise common.ParseException(
                    "not implemented extended UvMorph1")
        elif morph_type==5:
            # todo
            raise common.ParseException(
                    "not implemented extended UvMorph2")
        elif morph_type==6:
            # todo
            raise common.ParseException(
                    "not implemented extended UvMorph3")
        elif morph_type==7:
            # todo
            raise common.ParseException(
                    "not implemented extended UvMorph4")
        elif morph_type==8:
            morph.data=[self.read_material_morph_data()
                    for _ in range(offset_size)]
        else:
            raise common.ParseException(
                    "unknown morph type: {0}".format(morph_type))
        return morph

    def read_vertex_position_morph_offset(self):
        return pmx.VertexMorphOffset(
                self.read_vertex_index(), self.read_vector3())

    def read_material_morph_data(self):
        return pmx.MaterialMorphData(
                self.read_material_index(),
                self.read_int(1),
                self.read_rgba(),
                self.read_rgb(), self.read_float(),
                self.read_rgb(),
                self.read_rgba(),
                self.read_float(),
                self.read_rgba(),
                self.read_rgba(),
                self.read_rgba()
                )

    def read_display_slot(self):
        display_slot=pmx.DisplaySlot(self.read_text(), self.read_text(), 
                self.read_int(1))
        display_count=self.read_int(4)
        for _ in range(display_count):
            display_type=self.read_int(1)
            if display_type==0:
                display_slot.references.append(
                        (display_type, self.read_bone_index()))
            elif display_type==1:
                display_slot.references.append(
                        (display_type, self.read_morph_index()))
            else:
                raise common.ParseException(
                        "unknown display_type: {0}".format(display_type))
        return display_slot

    def read_rigidbody(self):
        return pmx.RigidBody(
                name=self.read_text(), 
                english_name=self.read_text(),
                bone_index=self.read_bone_index(),
                collision_group=self.read_int(1),
                no_collision_group=self.read_int(2),
                shape_type=self.read_int(1),
                shape_size=self.read_vector3(),
                shape_position=self.read_vector3(),
                shape_rotation=self.read_vector3(),
                mass=self.read_float(),
                linear_damping=self.read_float(),
                angular_damping=self.read_float(),
                restitution=self.read_float(),
                friction=self.read_float(),
                mode=self.read_int(1)
                )

    def read_joint(self):
        return pmx.Joint(
                name=self.read_text(),
                english_name=self.read_text(),
                joint_type=self.read_int(1),
                rigidbody_index_a=self.read_rigidbody_index(),
                rigidbody_index_b=self.read_rigidbody_index(),
                position=self.read_vector3(),
                rotation=self.read_vector3(),
                translation_limit_min=self.read_vector3(),
                translation_limit_max=self.read_vector3(),
                rotation_limit_min=self.read_vector3(),
                rotation_limit_max=self.read_vector3(),
                spring_constant_translation=self.read_vector3(),
                spring_constant_rotation=self.read_vector3())


def read_from_file(path):
    """
    read from file path, then return the pmx.Model.

    :Parameters:
      path
        file path

    >>> import pmx.reader
    >>> m=pmx.reader.read_from_file('resources/初音ミクVer2.pmx')
    >>> print(m)
    <pmx-2.0 "Miku Hatsune" 12354vertices>

    """
    pmx=read(io.BytesIO(common.readall(path)))
    pmx.path=path
    return pmx


def read(ios):
    """
    read from ios, then return the pmx pmx.Model.

    :Parameters:
      ios
        input stream (in io.IOBase)

    >>> import pmx.reader
    >>> m=pmx.reader.read(io.open('resources/初音ミクVer2.pmx', 'rb'))
    >>> print(m)
    <pmx-2.0 "Miku Hatsune" 12354vertices>

    """
    assert(isinstance(ios, io.IOBase))
    reader=common.BinaryReader(ios)

    # header
    signature=reader.unpack("4s", 4)
    if signature!=b"PMX ":
        raise common.ParseException(
                "invalid signature", signature)

    version=reader.read_float()
    if version!=2.0:
        print("unknown version", version)
    model=pmx.Model(version)

    # flags
    flag_bytes=reader.read_int(1)
    if flag_bytes!=8:
        raise common.ParseException(
                "invalid flag length", reader.flag_bytes)
    text_encoding=reader.read_int(1)
    extended_uv=reader.read_int(1)
    vertex_index_size=reader.read_int(1)
    texture_index_size=reader.read_int(1)
    material_index_size=reader.read_int(1)
    bone_index_size=reader.read_int(1)
    morph_index_size=reader.read_int(1)
    rigidbody_index_size=reader.read_int(1)
    
    # pmx custom reader
    reader=Reader(reader.ios,
            text_encoding,
            extended_uv,
            vertex_index_size,
            texture_index_size,
            material_index_size,
            bone_index_size,
            morph_index_size,
            rigidbody_index_size
            )

    # model info
    model.name = reader.read_text()
    model.english_name = reader.read_text()
    model.comment = reader.read_text()
    model.english_comment = reader.read_text()

    # model data
    model.vertices=[reader.read_vertex() 
            for _ in range(reader.read_int(4))]
    model.indices=[reader.read_vertex_index() 
            for _ in range(reader.read_int(4))]
    model.textures=[reader.read_text() 
            for _ in range(reader.read_int(4))]
    model.materials=[reader.read_material() 
            for _ in range(reader.read_int(4))]
    model.bones=[reader.read_bone() 
            for _ in range(reader.read_int(4))]
    model.morphs=[reader.read_morgh() 
            for _ in range(reader.read_int(4))]
    model.display_slots=[reader.read_display_slot() 
            for _ in range(reader.read_int(4))]
    model.rigidbodies=[reader.read_rigidbody()
            for _ in range(reader.read_int(4))]
    model.joints=[reader.read_joint()
            for _ in range(reader.read_int(4))]

    return model




# coding: utf-8
"""
VMDの読み込み
 * http://blog.goo.ne.jp/torisu_tetosuki/e/bc9f1c4d597341b394bd02b64597499d
 * http://yumin3123.at.webry.info/200810/article_4.html
 * http://atupdate.web.fc2.com/vmd_format.htm
"""
__author__="ousttrue"
__license__="zlib"
__versioon__="1.0.0"


import io
import os
import struct
from .. import common


class MorphFrame(object):
    """
    morphing animation data.
    """
    __slots__=['name', 'frame', 'ratio']
    def __init__(self, name):
        self.name=name
        self.frame=-1
        self.ratio=0

    def __cmp__(self, other):
        return cmp(self.frame, other.frame)


class BoneFrame(object):
    """
    bone animation data.
    """
    __slots__=['name', 'frame', 'pos', 'q', 'complement']
    def __init__(self, name):
        self.name=name
        self.frame=-1
        self.pos=common.Vector3()
        self.q=common.Quaternion()

    def __cmp__(self, other):
        return cmp(self.frame, other.frame)

    def __str__(self):
        return '<BoneFrame "%s" %d %s%s>' % (self.name, self.frame, self.pos, self.q)


class CameraFrame(object):
    """
    camera animation data.
    """
    __slots__=['frame', 'length', 'pos', 'euler', 'complement', 'angle', 'perspective']
    def __init__(self):
        self.frame=-1
        self.length=0
        self.pos=common.Vector3()
        self.euler=common.Vector3()
        self.complement=None
        self.angle=0
        self.perspective=True

    def __cmp__(self, other):
        return cmp(self.frame, other.frame)

    def __str__(self):
        return '<CameraFrame %d %s%s>' % (self.frame, self.pos, self.euler)


class Motion(object):
    __slots__=[
            'model_name',
            'motions',
            'shapes',
            'cameras',
            'lights',
            'last_frame',
            ]
    def __init__(self):
        self.model_name=''
        self.motions=[]
        self.shapes=[]
        self.cameras=[]
        self.lights=[]
        self.last_frame=0

    def __str__(self):
        return '<VMDLoader model: "%s", motion: %d, shape: %d, camera: %d, light: %d>' % (
            self.model_name, len(self.motions), len(self.shapes),
            len(self.cameras), len(self.lights))




# coding: utf-8
""" 
======================
Metasequioa MQO format
======================

file format
~~~~~~~~~~~
* http://www.metaseq.net/metaseq/format.html

specs
~~~~~
* textencoding: bytes(cp932)
* coordinate: right handed y-up
* uv origin: 
* face: edge(2), triangle(3), quadrangle(4)
* backculling: enable

"""

import os
import sys
import math
import warnings
from .. import common


"""
MQO loader
"""
class Material(object):
    """mqo material

    Attributes:
        name: cp932
        shader: 
        color: rgba
        diffuse:
        ambient:
        emit:
        specular:
        power:
        tex: cp932 windows file path
    """
    __slots__=[
            "name", "shader", "color", "diffuse", 
            "ambient", "emit", "specular", "power",
            "tex",
            ]
    def __init__(self, name):
        self.name=name
        self.shader=3
        self.color=common.RGBA(0.5, 0.5, 0.5, 1.0)
        self.diffuse=1.0
        self.ambient=0.0
        self.emit=0.0
        self.specular=0.0
        self.power=5.0
        self.tex=b""

    def getName(self): return self.name
    def getTexture(self): return self.tex

    def parse(self, line):
        offset=0
        while True:
            leftParenthesis=line.find(b"(", offset)
            if leftParenthesis==-1:
                break
            key=line[offset:leftParenthesis]
            rightParenthesis=line.find(b")", leftParenthesis+1)
            if rightParenthesis==-1:
                raise ValueError("assert")

            param=line[leftParenthesis+1:rightParenthesis]
            if key==b"shader":
                self.shader=int(param)
            elif key==b"col":
                self.color=common.RGBA(*[float(e) for e in param.split()])
            elif key==b"dif":
                self.diffuse=float(param)
            elif key==b"amb":
                self.ambient=float(param)
            elif key==b"emi":
                self.emit=float(param)
            elif key==b"spc":
                self.specular=float(param)
            elif key==b"power":
                self.power=float(param)
            elif key==b"tex":
                self.tex=param[1:-1]
            else:
                print(
                        "%s#parse" % self.name, 
                        "unknown key: %s" %  key
                        )

            offset=rightParenthesis+2

    def __str__(self):
        return "<Material %s shader: %d [%f, %f, %f, %f] %f>" % (
                self.name, self.shader,
                self.color[0], self.color[1], self.color[2], self.color[3],
                self.diffuse)


class Obj(object):
    """mqo object

    Attributes:
        name: cp932
        depth: object hierarchy 
        folding: 
        scale:
        rotation:
        translation:
        visible:
        locking:
        shading:
        facet: smoothing threshold
        color:
        color_type:
        mirror: mirroring
        mirror_axis:
        vertices:
        faces:
        edges:
        smoothing:
    """
    __slots__=["name", "depth", "folding", 
            "scale", "rotation", "translation",
            "visible", "locking", "shading", "facet",
            "color", "color_type", "mirror", "mirror_axis",
            "vertices", "faces", "edges", "smoothing",
            ]

    def __init__(self, name):
        self.name=name
        self.vertices=[]
        self.faces=[]
        self.edges=[]
        self.depth=0
        self.folding=0
        self.scale=[1, 1, 1]
        self.rotation=[0, 0, 0]
        self.translation=[0, 0, 0]
        self.visible=15
        self.locking=0
        self.shading=0
        self.facet=59.5
        self.color=[1, 1, 1]
        self.color_type=0
        self.mirror=0
        self.smoothing=0

    def getName(self): return self.name

    def addVertex(self, x, y, z):
        self.vertices.append(common.Vector3(x, y, z))

    def addFace(self, face):
        if face.index_count==2:
            self.edges.append(face)
        else:
            self.faces.append(face)

    def __str__(self):
        return "<Object %s, %d vertices, %d faces>" % (
                self.name, len(self.vertices), len(self.faces))


class Face(object):
    """mqo face

    Attributes:
        index_count: 2 or 3 or 4
        indices: index x index_count
        material_index:
        col: vertex_color x index_count
        uv: Vector2 x index_count
    """
    __slots__=[
            "index_count",
            "indices", "material_index", "col", "uv",
            ]
    def __init__(self, index_count, line):
        if index_count<2 or index_count>4:
            raise ValueError("invalid vertex count: %d" % index_count)
        self.material_index=0
        self.col=[]
        self.uv=[common.Vector2(0, 0)]*4
        self.index_count=index_count
        offset=0
        while True:
            leftParenthesis=line.find(b"(", offset)
            if leftParenthesis==-1:
                break
            key=line[offset:leftParenthesis]
            rightParenthesis=line.find(b")", leftParenthesis+1)
            if rightParenthesis==-1:
                raise ValueError("assert")
            params=line[leftParenthesis+1:rightParenthesis].split()
            if key==b"V":
                self.indices=[int(e) for e in params]
            elif key==b"M":
                self.material_index=int(params[0])
            elif key==b"UV":
                uv_list=[float(e) for e in params]
                self.uv=[]
                for i in range(0, len(uv_list), 2):
                    self.uv.append(common.Vector2(uv_list[i], uv_list[i+1]))
            elif key==b"COL":
                for n in params:
                    d=int(n)
                    # R
                    d, m=divmod(d, 256)
                    self.col.append(m)
                    # G
                    d, m=divmod(d, 256)
                    self.col.append(m)
                    # B
                    d, m=divmod(d, 256)
                    self.col.append(m)
                    # A
                    d, m=divmod(d, 256)
                    self.col.append(m)
            else:
                print("Face#__init__:unknown key: %s" % key)

            offset=rightParenthesis+2

    def getIndex(self, i): return self.indices[i]
    def getUV(self, i): return self.uv[i] if i<len(self.uv) else common.Vector2(0, 0)


class Model(object):
    def __init__(self):
        self.has_mikoto=False
        self.materials=[]
        self.objects=[]





# coding: utf-8
from .. import bl
from ..pymeshio import englishmap


class IKSolver(object):
    __slots__=['target', 'effector', 'length', 'iterations', 'weight']
    def __init__(self, target, effector, length, iterations, weight):
        self.target=target
        self.effector=effector
        self.length=length
        self.iterations=iterations
        self.weight=weight


class Bone(object):
    __slots__=['index', 'name', 'ik_index',
            'pos', 'tail', 'parent_index', 'tail_index', 'type', 'isConnect']
    def __init__(self, name, pos, tail, isConnect):
        self.index=-1
        self.name=name
        self.pos=pos
        self.tail=tail
        self.parent_index=None
        self.tail_index=None
        self.type=0
        self.isConnect=isConnect
        self.ik_index=0

    def __eq__(self, rhs):
        return self.index==rhs.index

    def __str__(self):
        return "<Bone %s %d>" % (self.name, self.type)


class BoneBuilder(object):
    __slots__=['bones', 'boneMap', 'ik_list', 'bone_groups',]
    def __init__(self):
        self.bones=[]
        self.boneMap={}
        self.ik_list=[]
        self.bone_groups=[]

    def getBoneGroup(self, bone):
        for i, g in enumerate(self.bone_groups):
            for b in g[1]:
                if b==bone.name:
                    return i+1
        print('no gorup', bone)
        return 0

    def build(self, armatureObj):
        if not armatureObj:
            return

        bl.message("build skeleton")
        armature=bl.object.getData(armatureObj)

        ####################
        # bone group
        ####################
        for g in bl.object.boneGroups(armatureObj):
            self.bone_groups.append((g.name, []))

        ####################
        # get bones
        ####################
        for b in armature.bones.values():
            if not b.parent:
                # root bone
                bone=Bone(b.name, 
                        bl.bone.getHeadLocal(b),
                        bl.bone.getTailLocal(b),
                        False)
                self.__addBone(bone)
                self.__getBone(bone, b)

        for b in armature.bones.values():
            if not b.parent:
                self.__checkConnection(b, None)

        ####################
        # get IK
        ####################
        pose = bl.object.getPose(armatureObj)
        for b in pose.bones.values():
            ####################
            # assing bone group
            ####################
            self.__assignBoneGroup(b, b.bone_group)
            for c in b.constraints:
                if bl.constraint.isIKSolver(c):
                    ####################
                    # IK target
                    ####################
                    target=self.__boneByName(bl.constraint.ikTarget(c))
                    target.type=2

                    ####################
                    # IK effector
                    ####################
                    # IK 接続先
                    link=self.__boneByName(b.name)
                    link.type=6

                    # IK chain
                    e=b.parent
                    chainLength=bl.constraint.ikChainLen(c)
                    for i in range(chainLength):
                        # IK影響下
                        chainBone=self.__boneByName(e.name)
                        chainBone.type=4
                        chainBone.ik_index=target.index
                        e=e.parent
                    self.ik_list.append(
                            IKSolver(target, link, chainLength, 
                                int(bl.constraint.ikItration(c) * 0.1), 
                                bl.constraint.ikRotationWeight(c)
                                ))

        ####################

        # boneのsort
        self._sortBy()
        self._fix()
        # IKのsort
        def getIndex(ik):
            for i, v in enumerate(englishmap.boneMap):
                if v[0]==ik.target.name:
                    return i
            return len(englishmap.boneMap)
        self.ik_list.sort(key=getIndex)

    def __assignBoneGroup(self, poseBone, boneGroup):
        if boneGroup:
            for g in self.bone_groups:
                if g[0]==boneGroup.name:
                    g[1].append(poseBone.name)

    def __checkConnection(self, b, p):
        if bl.bone.isConnected(b):
            parent=self.__boneByName(p.name)
            parent.isConnect=True

        for c in b.children:
            self.__checkConnection(c, b)

    def _sortBy(self):
        """
        boneMap順に並べ替える
        """
        boneMap=englishmap.boneMap
        original=self.bones[:]
        def getIndex(bone):
            for i, k_v in enumerate(boneMap):
                if k_v[0]==bone.name:
                    return i
            #print(bone)
            return len(boneMap)

        self.bones.sort(key=getIndex)

        sortMap={}
        for i, b in enumerate(self.bones):
            src=original.index(b)
            sortMap[src]=i
        for b in self.bones:
            b.index=sortMap[b.index]
            if b.parent_index:
                b.parent_index=sortMap[b.parent_index]
            if b.tail_index:
                b.tail_index=sortMap[b.tail_index]
            if b.ik_index>0:
                b.ik_index=sortMap[b.ik_index]

    def _fix(self):
        """
        調整
        """
        for b in self.bones:
            # parent index
            if b.parent_index==None:
                b.parent_index=-1
            else:
                if b.type==6 or b.type==7:
                    # fix tail bone
                    parent=self.bones[b.parent_index]
                    #print('parnet', parent.name)
                    parent.tail_index=b.index

        for b in self.bones:
            if b.tail_index==None:
                b.tail_index=0
            elif b.type==9:
                b.tail_index==0

    def getIndex(self, bone):
        for i, b in enumerate(self.bones):
            if b==bone:
                return i
        assert(false)

    def indexByName(self, name):
        if name=='':
            return 0
        else:
            try:
                return self.getIndex(self.__boneByName(name))
            except:
                return 0

    def __boneByName(self, name):
        return self.boneMap[name]

    def __getBone(self, parent, b):
        if len(b.children)==0:
            parent.type=7
            return

        for i, c in enumerate(b.children):
            bone=Bone(c.name, 
                    bl.bone.getHeadLocal(c),
                    bl.bone.getTailLocal(c),
                    bl.bone.isConnected(c))
            self.__addBone(bone)
            if parent:
                bone.parent_index=parent.index
                #if i==0:
                if bone.isConnect or (not parent.tail_index and parent.tail==bone.pos):
                    parent.tail_index=bone.index
            self.__getBone(bone, c)

    def __addBone(self, bone):
        bone.index=len(self.bones)
        self.bones.append(bone)
        self.boneMap[bone.name]=bone




# coding: utf-8
"""
========================
MikuMikuDance PMD format
========================

file format
~~~~~~~~~~~
* http://blog.goo.ne.jp/torisu_tetosuki/e/209ad341d3ece2b1b4df24abf619d6e4 

specs
~~~~~
* textencoding: bytes(cp932)
* coordinate: left handed y-up(DirectX)
* uv origin: 
* face: only triangle
* backculling: 

"""
import os
import sys
import struct
import warnings
from .. import common


class Vertex(object):
    """
    ==========
    pmd vertex
    ==========
    two bone weighted vertex with normal and uv.

    format
    ~~~~~~
    * http://blog.goo.ne.jp/torisu_tetosuki/e/5a1b16e2f61067838dfc66d010389707

    :IVariables:
        pos
            Vector3
        normal 
            Vector3
        uv 
            Vector2
        bone0 
            bone index
        bone1 
            bone index
        weight0 
            bone0 influence.  min: 0, max: 100
        edge_flag 
            int flag.  0: edge on, 1: edge off
    """
    __slots__=['pos', 'normal', 'uv', 'bone0', 'bone1', 'weight0', 'edge_flag']
    def __init__(self, pos, normal, uv, 
            bone0, bone1, weight0, edge_flag):
        self.pos=pos
        self.normal=normal
        self.uv=uv
        self.bone0=bone0
        self.bone1=bone1
        self.weight0=weight0
        self.edge_flag=edge_flag

    def __str__(self):
        return "<%s %s %s, (%d, %d, %d)>" % (
                str(self.pos), 
                str(self.normal), 
                str(self.uv), 
                self.bone0, self.bone1, self.weight0)

    def __eq__(self, rhs):
        return (
                self.pos==rhs.pos
                and self.normal==rhs.normal
                and self.uv==rhs.uv
                and self.bone0==rhs.bone0
                and self.bone1==rhs.bone1
                and self.weight0==rhs.weight0
                and self.edge_flag==rhs.edge_flag
                )

    def __getitem__(self, key):
        if key==0:
            return self.pos.x
        elif key==1:
            return self.pos.y
        elif key==2:
            return self.pos.z
        else:
            assert(False)


class Material(object):
    """
    ============
    pmd material
    ============

    format
    ~~~~~~
    * http://blog.goo.ne.jp/torisu_tetosuki/e/ea0bb1b1d4c6ad98a93edbfe359dac32

    :IVariables:
        diffuse_color
            RGB
        alpha
            float
        specular_factor
            float
        specular_color
            RGB
        ambient_color
            RGB
        toon_index
            int
        edge_flag
            int
        vertex_count
            indices length
        texture_file
            texture file path
    """
    __slots__=[
            'diffuse_color', 'alpha', 
            'specular_factor', 'specular_color', 'ambient_color', 
            'toon_index', 'edge_flag',
            'vertex_count', 'texture_file', 
            ]
    def __init__(self, diffuse_color, alpha,
            specular_factor, specular_color, ambient_color,
            toon_index, edge_flag, vertex_count, texture_file):
        self.diffuse_color=diffuse_color
        self.alpha=alpha
        self.specular_factor=specular_factor
        self.specular_color=specular_color
        self.ambient_color=ambient_color
        self.toon_index=toon_index
        self.edge_flag=edge_flag
        self.vertex_count=vertex_count
        self.texture_file=texture_file

    def __str__(self):
        return "<Material [%f, %f, %f, %f]>" % (
                self.diffuse[0], self.diffuse[1], 
                self.diffuse[2], self.diffuse[3],
                )

    def __eq__(self, rhs):
        return (
                self.diffuse_color==rhs.diffuse_color
                and self.alpha==rhs.alpha
                and self.specular_factor==rhs.specular_factor
                and self.specular_color==rhs.specular_color
                and self.ambient_color==rhs.ambient_color
                and self.toon_index==rhs.toon_index
                and self.edge_flag==rhs.edge_flag
                and self.vertex_count==rhs.vertex_count
                and self.texture_file==rhs.texture_file
                )


class Bone(object):
    """
    ==========
    pmd bone
    ==========

    format
    ~~~~~~
    * http://blog.goo.ne.jp/torisu_tetosuki/e/638463f52d0ad6ca1c46fd315a9b17d0

    :IVariables:
        name 
            bone name
        english_name
            bone english_name
        index
            boen index(append for internal use)
        type
            bone type
        ik
            ik(append for internal use)
        pos
            bone head position
        ik_index
            ik target bone index
        parent_index
            parent bone index
        tail_index
            tail bone index
        parent
            parent bone(append for internal use)
        tail
            tail bone(append for internal use)
        children
            children bone(append for internal use)
    """
    # kinds
    ROTATE = 0
    ROTATE_MOVE = 1
    IK = 2
    IK_ROTATE_INFL = 4
    ROTATE_INFL = 5
    IK_TARGET = 6
    UNVISIBLE = 7
    # since v4.0
    ROLLING=8 # ?
    TWEAK=9
    __slots__=['name', 'index', 'type', 'parent', 'ik', 'pos',
            'children', 'english_name', 'ik_index',
            'parent_index', 'tail_index', 'tail',
            ]
    def __init__(self, name=b'bone', type=0):
        self.name=name
        self.index=0
        self.type=type
        self.parent_index=0xFFFF
        self.tail_index=0
        self.tail=common.Vector3(0, 0, 0)
        self.parent=None
        self.ik_index=0xFFFF
        self.pos=common.Vector3(0, 0, 0)
        self.children=[]
        self.english_name=b''

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.index==rhs.index
                and self.type==rhs.type
                and self.parent_index==rhs.parent_index
                and self.tail_index==rhs.tail_index
                and self.tail==rhs.tail
                and self.ik_index==rhs.ik_index
                and self.pos==rhs.pos
                and self.children==rhs.children
                and self.english_name==rhs.english_name
                )

    def hasParent(self):
        return self.parent_index!=0xFFFF

    def hasChild(self):
        return self.tail_index!=0 and self.tail_index!=0xFFFF

    def display(self, indent=None):
        indent=indent or []
        if len(indent)>0:
            prefix=''
            for i, is_end in enumerate(indent):
                if i==len(indent)-1:
                    break
                else:
                    prefix+='  ' if is_end else ' |'
            uni='%s +%s(%s)' % (prefix, unicode(self), self.english_name)
            print(uni.encode(ENCODING))
        else:
            uni='%s(%s)' % (unicode(self), self.english_name)
            print(uni.encode(ENCODING))

        child_count=len(self.children)
        for i in range(child_count):
            child=self.children[i]
            if i<child_count-1:
                child.display(indent+[False])
            else:
                # last
                child.display(indent+[True])

# 0
class Bone_Rotate(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_Rotate, self).__init__(name, 0)
    def __str__(self):
        return '<ROTATE %s>' % (self.name)
# 1
class Bone_RotateMove(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_RotateMove, self).__init__(name, 1)
    def __str__(self):
        return '<ROTATE_MOVE %s>' % (self.name)
# 2
class Bone_IK(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_IK, self).__init__(name, 2)
    def __str__(self):
        return '<IK %s>' % (self.name)
# 4
class Bone_IKRotateInfl(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_IKRotateInfl, self).__init__(name, 4)
    def __str__(self):
        return '<IK_ROTATE_INFL %s>' % (self.name)
# 5
class Bone_RotateInfl(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_RotateInfl, self).__init__(name, 5)
    def __str__(self):
        return '<ROTATE_INFL %s>' % (self.name)
# 6
class Bone_IKTarget(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_IKTarget, self).__init__(name, 6)
    def __str__(self):
        return '<IK_TARGET %s>' % (self.name)
# 7
class Bone_Unvisible(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_Unvisible, self).__init__(name, 7)
    def __str__(self):
        return '<UNVISIBLE %s>' % (self.name)
# 8
class Bone_Rolling(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_Rolling, self).__init__(name, 8)
    def __str__(self):
        return '<ROLLING %s>' % (self.name)
# 9
class Bone_Tweak(Bone):
    __slots__=[]
    def __init__(self, name):
        super(Bone_Tweak, self).__init__(name, 9)
    def __str__(self):
        return '<TWEAK %s>' % (self.name)


def createBone(name, type):
    if type==0:
        return Bone_Rotate(name)
    elif type==1:
        return Bone_RotateMove(name)
    elif type==2:
        return Bone_IK(name)
    elif type==3:
        raise Exception("no used bone type: 3(%s)" % name)
    elif type==4:
        return Bone_IKRotateInfl(name)
    elif type==5:
        return Bone_RotateInfl(name)
    elif type==6:
        return Bone_IKTarget(name)
    elif type==7:
        return Bone_Unvisible(name)
    elif type==8:
        return Bone_Rolling(name)
    elif type==9:
        return Bone_Tweak(name)
    else:
        raise Exception("unknown bone type: %d(%s)" % (type, name.decode('cp932')))


class IK(object):
    __slots__=['index', 'target', 'iterations', 'weight', 'length', 'children']
    def __init__(self, index=0, target=0):
        self.index=index
        self.target=target
        self.iterations=None
        self.weight=None
        self.children=[]

    def __str__(self):
        return "<IK index: %d, target: %d, iterations: %d, weight: %f, children: %s(%d)>" %(self.index, self.target, self.iterations, self.weight, '-'.join([str(i) for i in self.children]), len(self.children))

    def __eq__(self, rhs):
        return (
                self.index==rhs.index
                and self.target==rhs.target
                and self.iterations==rhs.iterations
                and self.weight==rhs.weight
                and self.children==rhs.children
                )


class Morph(object):
    __slots__=['name', 'type', 'indices', 'pos_list', 'english_name',
            'vertex_count']
    def __init__(self, name):
        self.name=name
        self.type=None
        self.indices=[]
        self.pos_list=[]
        self.english_name=b''
        self.vertex_count=0

    def append(self, index, x, y, z):
        self.indices.append(index)
        self.pos_list.append(common.Vector3(x, y, z))

    def __str__(self):
        return '<Skin name: "%s", type: %d, vertex: %d>' % (
            self.name, self.type, len(self.indices))

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.type==rhs.type
                and self.indices==rhs.indices
                and self.pos_list==rhs.pos_list
                and self.english_name==rhs.english_name
                and self.vertex_count==rhs.vertex_count
                )


class BoneGroup(object):
    __slots__=['name', 'english_name']
    def __init__(self, name=b'group', english_name=b'center'): 
        self.name=name
        self.english_name=english_name

    def __eq__(self, rhs):
        return self.name==rhs.name and self.english_name==rhs.english_name


SHAPE_SPHERE=0
SHAPE_BOX=1
SHAPE_CAPSULE=2

RIGIDBODY_KINEMATICS=0
RIGIDBODY_PHYSICS=1
RIGIDBODY_PHYSICS_WITH_BONE=2


class RigidBody(object):
    __slots__=['name', 
            'bone_index', 
            'collision_group', 
            'no_collision_group', 
            'shape_type',
            'shape_size',
            'shape_position', 
            'shape_rotation', 
            'mass',
            'linear_damping', 
            'angular_damping', 
            'restitution', 
            'friction', 
            'mode'
            ]
    def __init__(self, name,
            bone_index, 
            collision_group, 
            no_collision_group, 
            shape_type,
            shape_size,
            shape_position, 
            shape_rotation,
            mass,
            linear_damping, 
            angular_damping, 
            restitution, 
            friction, 
            mode
            ):
        self.name=name
        self.bone_index=bone_index
        self.collision_group=collision_group 
        self.no_collision_group=no_collision_group 
        self.shape_type=shape_type
        self.shape_size=shape_size
        self.shape_position=shape_position
        self.shape_rotation=shape_rotation
        self.mass=mass
        self.linear_damping=linear_damping
        self.angular_damping=angular_damping
        self.restitution=restitution
        self.friction=friction
        self.mode=mode

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.bone_index==rhs.bone_index
                and self.collision_group==rhs.collision_group
                and self.no_collision_group==rhs.no_collision_group
                and self.shape_type==rhs.shape_type
                and self.shape_size==rhs.shape_size
                and self.shape_position==rhs.shape_position
                and self.shape_rotation==rhs.shape_rotation
                and self.mass==rhs.mass
                and self.linear_damping==rhs.linear_damping
                and self.angular_damping==rhs.angular_damping
                and self.restitution==rhs.restitution
                and self.friction==rhs.friction
                and self.mode==rhs.mode
                )


class Joint(object):
    __slots__=[ 'name', 'rigidbody_index_a', 'rigidbody_index_b', 
            'position', 'rotation',
            'translation_limit_max', 'translation_limit_min',
            'rotation_limit_max', 'rotation_limit_min',
            'spring_constant_translation', 'spring_constant_rotation',
            ]
    def __init__(self, name,
            rigidbody_index_a, rigidbody_index_b,
            position, rotation,
            translation_limit_max, translation_limit_min,
            rotation_limit_max, rotation_limit_min,
            spring_constant_translation, spring_constant_rotation
            ):
        self.name=name
        self.rigidbody_index_a=rigidbody_index_a
        self.rigidbody_index_b=rigidbody_index_b
        self.position=position
        self.rotation=rotation
        self.translation_limit_max=translation_limit_max
        self.translation_limit_min=translation_limit_min
        self.rotation_limit_max=rotation_limit_max
        self.rotation_limit_min=rotation_limit_min
        self.spring_constant_translation=spring_constant_translation
        self.spring_constant_rotation=spring_constant_rotation

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.rigidbody_index_a==rhs.rigidbody_index_a
                and self.rigidbody_index_b==rhs.rigidbody_index_b
                and self.position==rhs.position
                and self.rotation==rhs.rotation
                and self.translation_limit_max==rhs.translation_limit_max
                and self.translation_limit_min==rhs.translation_limit_min
                and self.rotation_limit_max==rhs.rotation_limit_max
                and self.rotation_limit_min==rhs.rotation_limit_min
                and self.spring_constant_translation==rhs.spring_constant_translation
                and self.spring_constant_rotation==rhs.spring_constant_rotation
                )


class Model(object):
    """pmd loader class.

    Attributes:
        io: internal use.
        end: internal use.
        pos: internal user.

        version: pmd version number
        _name: internal
    """
    __slots__=[
            'path',
            'version', 'name', 'comment',
            'english_name', 'english_comment',
            'vertices', 'indices', 'materials', 'bones', 
            'ik_list', 'morphs',
            'morph_indices', 'bone_group_list', 'bone_display_list',
            'toon_textures',
            'rigidbodies', 'joints',

            'no_parent_bones',
            ]
    def __init__(self, version=1.0):
        self.path=b''
        self.version=version
        self.name=b''
        self.comment=b''
        self.english_name=b''
        self.english_comment=b''
        self.vertices=[]
        self.indices=[]
        self.materials=[]
        self.bones=[]
        self.ik_list=[]
        self.morphs=[]
        self.morph_indices=[]
        self.bone_group_list=[]
        self.bone_display_list=[]
        # extend
        self.toon_textures=[b'']*10
        self.rigidbodies=[]
        self.joints=[]
        # innner use
        self.no_parent_bones=[]

    def each_vertex(self): return self.vertices
    def getUV(self, i): return self.vertices[i].uv

    def __str__(self):
        return '<pmd-%g, "%s" vertex: %d, face: %d, material: %d, bone: %d ik: %d, skin: %d>' % (
            self.version, self.name, len(self.vertices), len(self.indices),
            len(self.materials), len(self.bones), len(self.ik_list), len(self.morphs))

    def __eq__(self, rhs):
        return (
                self.name==rhs.name
                and self.comment==rhs.comment
                and self.english_name==rhs.english_name
                and self.english_comment==rhs.english_comment
                and self.vertices==rhs.vertices
                and self.indices==rhs.indices
                and self.materials==rhs.materials
                and self.bones==rhs.bones
                and self.ik_list==rhs.ik_list
                and self.morphs==rhs.morphs
                and self.morph_indices==rhs.morph_indices
                and self.bone_group_list==rhs.bone_group_list
                and self.bone_display_list==rhs.bone_display_list
                and self.toon_textures==rhs.toon_textures
                and self.rigidbodies==rhs.rigidbodies
                and self.joints==rhs.joints
                )

    def diff(self, rhs):
        diffrence=[]
        for attr in [
                'name', 'comment', 'english_name', 'english_comment',
                'vertices', 'indices', 'materials', 
                'bones', 'ik_list',
                'morphs', 
                'morph_indices', 'bone_group_list', 'bone_display_list', 
                'toon_textures', 
                'rigidbodies', 'joints',
                ]:
            if getattr(self, attr)!=getattr(rhs, attr):
                diffrence.append(attr)
        if len(diffrence)>0:
            print(diffrence)




#!/usr/bin/pyhthon
# coding: utf-8

from OpenGL.GL import *
from . import texture

'''
Material

* 色
'''
class Material(object):
    def __init__(self, r, g, b, a):
        self.r=r
        self.g=g
        self.b=b
        self.a=a

    def begin(self):
        glColor4f(self.r, self.g, self.b, self.a)

    def end(self):
        pass

    def onInitialize(self):
        pass

    @staticmethod
    def create(src):
        m=material.Material(*src.col)
        return m


'''
Material

* 色
* テクスチャー
'''
class MQOMaterial(object):
    def __init__(self):
        self.rgba=(1, 1, 1, 1)
        self.vcol=False
        self.texture=None

    def __enter__(self):
        self.begin()

    def __exit__(self):
        self.end()

    def begin(self):
        glColor4f(*self.rgba)
        if self.texture:
            self.texture.begin()

        # backface culling
        glEnable(GL_CULL_FACE)
        glFrontFace(GL_CW)
        glCullFace(GL_BACK)
        # alpha test
        glEnable(GL_ALPHA_TEST);
        glAlphaFunc(GL_GREATER, 0.5);

    def end(self):
        if self.texture:
            self.texture.end()

    def onInitialize(self):
        pass
        #if self.texture:
        #    self.texture.onInitialize()




#!/usr/bin/env python
# coding: utf-8
"""
this script require pyOpenGL, PIL, numpy and Togl(tck/tk).

Togl install on Windows
=======================

* download Togl2.0-8.4-Windows.zip
* copy Togl2.0-8.4-Windows/lib/Togl2.0 to C:/PythonXX/tcl/Togl2.0
"""

import sys
import os
try:
    import tkinter
    import tkinter.filedialog as tkinter_filedialog
except ImportError as e:
    import Tkinter as tkinter
    import tkFileDialog as tkinter_filedialog
import togl
import opengl
import opengl.rokuro
import mqobuilder
import pmdbuilder
import pmxbuilder


class Frame(tkinter.Frame):
    def __init__(self, width, height, master=None, **kw):
        #super(Frame, self).__init__(master, **kw)
        tkinter.Frame.__init__(self, master, **kw)
        self.master.title('pymeshio viewer')
        self.current='.'
        # setup menu
        menu_bar = tkinter.Menu(self)
        self.master.config(menu=menu_bar)

        menu_file = tkinter.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=menu_file, underline=0)

        menu_file.add_command(label='Open', under=0, command=self.onOpen)

        # setup opengl widget
        self.view=opengl.rokuro.RokuroView()
        self.glworld=opengl.BaseController(self.view)
        self.glwidget=togl.Widget(self, self.glworld, width=width, height=height)
        self.glwidget.pack(fill=tkinter.BOTH, expand=True)

        # event binding
        self.bind('<Key>', self.onKeyDown)
        self.bind('<MouseWheel>', lambda e: self.glworld.onWheel(-e.delta) and self.glwidget.onDraw())

    def onOpen(self):
        path=tkinter_filedialog.askopenfilename(
                filetypes=[
                    ('poloygon model files', '*.mqo;*.pmd;*.pmx'),
                    ], 
                initialdir=self.current)
        self.current=os.path.dirname(path)
        self.load(path)

    def load(self, path):
        model=self.loadModel(path)
        if not model:
            print('fail to load %s' % path)
            return
        print('load %s' % path)
        print(model)
        self.glworld.setRoot(model)
        bb=model.get_boundingbox()
        print(bb)
        self.view.look_bb(*bb)
        self.glwidget.onDraw()

    def loadModel(self, path):
        if path.lower().endswith(".mqo"):
            return mqobuilder.build(path)
        elif path.lower().endswith(".pmd"):
            return pmdbuilder.build(path)
        elif path.lower().endswith(".pmx"):
            return pmxbuilder.build(path)
        else:
            print("unknown file format: {0}".format(path))

    def onKeyDown(self, event):
        key=event.keycode
        if key==27:
            # Escape
            sys.exit()
        if key==81:
            # q
            sys.exit()
        else:
            print("keycode: %d" % key)


if __name__ == '__main__':
    f = Frame(width=600, height=600)
    f.pack(fill=tkinter.BOTH, expand=True)
    f.focus_set()
    if len(sys.argv)>1:
        f.load(sys.argv[1])
    f.mainloop()




# coding: utf-8

bl_info={
        'category': 'Import-Export',
        'name': 'meshio. (.pmd)(.pmx)(.mqo)',
        'author': 'ousttrue',
        'blender': (2, 6, 0),
        'location': 'File > Import-Export',
        'description': 'Import-Export PMD/PMX/MQO meshes',
        'warning': 'pmx importer/exporter is under development', 
        'wiki_url': 'http://meshio.sourceforge.jp/',
        'support': 'COMMUNITY',
        }


# To support reload properly, try to access a package var, if it's there, reload everything
if 'bpy' in locals():
    import imp
    def reload_module(name):
        if name in locals():
            imp.reaload(locals()[name])
    reload_module('import_pmx')
    reload_module('export_pmx')
    reload_module('import_pmd')
    reload_module('export_pmd')
    reload_module('import_mqo')
    reload_module('export_mqo')
   

import bpy
import bpy_extras.io_utils
from . import bl


class ImportPmd(bpy.types.Operator, bpy_extras.io_utils.ImportHelper):
    '''Import from PMD file format (.pmd)'''
    bl_idname = 'import_scene.mmd_pmd'
    bl_label = 'Import PMD'
    bl_options={'UNDO'}
    filename_ext = '.pmd'
    filter_glob = bpy.props.StringProperty(
            default='*.pmd', options={'HIDDEN'})

    def execute(self, context):
        from . import import_pmd
        bl.initialize('pmd_import', context.scene)
        import_pmd._execute(**self.as_keywords(
            ignore=('filter_glob',)))
        bl.finalize()
        return {'FINISHED'}

    @classmethod
    def menu_func(klass, self, context):
        self.layout.operator(klass.bl_idname,
                text='MikuMikuDance model (.pmd)',
                icon='PLUGIN'
                )


class ImportPmx(bpy.types.Operator, bpy_extras.io_utils.ImportHelper):
    '''Import from PMX Format(.pmx)'''
    bl_idname='import_scene.mmd_pmx'
    bl_label='Import PMX'
    bl_options={'UNDO'}
    filename_ext='.pmx'
    filter_glob=bpy.props.StringProperty(
            default='*.pmx', options={'HIDDEN'})

    def execute(self, context):
        from . import import_pmx
        bl.initialize('pmd_import', context.scene)
        import_pmx._execute(**self.as_keywords(
            ignore=('filter_glob',)))
        bl.finalize()
        return  {'FINISHED'}

    @classmethod
    def menu_func(klass, self, context):
        self.layout.operator(klass.bl_idname, 
                text='MikuMikuDance model (.pmx)',
                icon='PLUGIN'
                )


class ImportMqo(bpy.types.Operator, bpy_extras.io_utils.ImportHelper):
    '''Import from MQO file format (.mqo)'''
    bl_idname = 'import_scene.metasequioa_mqo'
    bl_label = 'Import MQO'
    bl_options={'UNDO'}
    filename_ext = '.mqo'
    filter_glob = bpy.props.StringProperty(
            default='*.mqo', options={'HIDDEN'})

    scale = bpy.props.FloatProperty(
            name='Scale',
            description='Scale the MQO by this value',
            min=0.0001, max=1000000.0,
            soft_min=0.001, soft_max=100.0, default=0.1)

    def execute(self, context):
        from . import import_mqo
        bl.initialize('mqo_import', context.scene)
        import_mqo._execute(**self.as_keywords(
            ignore=('filter_glob',)))
        bl.finalize()
        return {'FINISHED'}

    @classmethod
    def menu_func(klass, self, context):
        self.layout.operator(klass.bl_idname,
                text="Metasequoia (.mqo)",
                icon='PLUGIN'
                )


class ExportPmd(bpy.types.Operator, bpy_extras.io_utils.ExportHelper):
    '''Export to PMD file format (.pmd)'''
    bl_idname = 'export_scene.mmd_pmd'
    bl_label = 'Export PMD'

    filename_ext = '.pmd'
    filter_glob = bpy.props.StringProperty(
            default='*.pmd', options={'HIDDEN'})

    use_selection = bpy.props.BoolProperty(
            name='Selection Only', 
            description='Export selected objects only', 
            default=False)

    def execute(self, context):
        from . import export_pmd
        bl.initialize('pmd_export', context.scene)
        export_pmd._execute(**self.as_keywords(
            ignore=('check_existing', 'filter_glob', 'use_selection')))
        bl.finalize()
        return {'FINISHED'}

    @classmethod
    def menu_func(klass, self, context):
        default_path=bpy.data.filepath.replace('.blend', '.pmd')
        self.layout.operator(klass.bl_idname,
                text='Miku Miku Dance Model(.pmd)',
                icon='PLUGIN'
                ).filepath=default_path


class ExportPmx(bpy.types.Operator, bpy_extras.io_utils.ExportHelper):
    '''Export to PMX file format (.pmx)'''
    bl_idname = 'export_scene.mmd_pmx'
    bl_label = 'Export PMX'

    filename_ext = '.pmx'
    filter_glob = bpy.props.StringProperty(
            default='*.pmx', options={'HIDDEN'})

    use_selection = bpy.props.BoolProperty(
            name='Selection Only', 
            description='Export selected objects only', 
            default=False)

    def execute(self, context):
        from . import export_pmx
        bl.initialize('pmx_export', context.scene)
        export_pmx._execute(**self.as_keywords(
            ignore=('check_existing', 'filter_glob', 'use_selection')))
        bl.finalize()
        return {'FINISHED'}

    @classmethod
    def menu_func(klass, self, context):
        default_path=bpy.data.filepath.replace('.blend', '.pmx')
        self.layout.operator(klass.bl_idname,
                text='Miku Miku Dance Model(.pmx)',
                icon='PLUGIN'
                ).filepath=default_path


class ExportMqo(bpy.types.Operator, bpy_extras.io_utils.ExportHelper):
    '''Save a Metasequoia MQO file.'''
    bl_idname = 'export_scene.metasequioa_mqo'
    bl_label = 'Export MQO'

    filename_ext = '.mqo'
    filter_glob = bpy.props.StringProperty(
            default='*.mqo', options={'HIDDEN'})

    use_selection = bpy.props.BoolProperty(
            name='Selection Only', 
            description='Export selected objects only', 
            default=False)

    scale = bpy.props.FloatProperty(
            name='Scale',
            description='Scale the MQO by this value',
            min=0.0001, max=1000000.0,
            soft_min=0.001, soft_max=100.0, default=10.0)

    apply_modifier = bpy.props.BoolProperty(
            name='ApplyModifier',
            description='Would apply modifiers',
            default=False)

    def execute(self, context):
        from . import export_mqo
        bl.initialize('mqo_export', context.scene)
        export_mqo._execute(**self.as_keywords(
            ignore=('check_existing', 'filter_glob', 'use_selection')))
        bl.finalize()
        return {'FINISHED'}

    @classmethod
    def menu_func(klass, self, context):
        default_path=bpy.data.filepath.replace('.blend', '.mqo')
        self.layout.operator(klass.bl_idname,
                text='Metasequoia (.mqo)',
                icon='PLUGIN'
                ).filepath=default_path


def register():
    bpy.utils.register_module(__name__)
    bpy.types.INFO_MT_file_import.append(ImportPmd.menu_func)
    bpy.types.INFO_MT_file_import.append(ImportPmx.menu_func)
    bpy.types.INFO_MT_file_import.append(ImportMqo.menu_func)
    bpy.types.INFO_MT_file_export.append(ExportPmd.menu_func)
    bpy.types.INFO_MT_file_export.append(ExportPmx.menu_func)
    bpy.types.INFO_MT_file_export.append(ExportMqo.menu_func)

def unregister():
    bpy.utils.unregister_module(__name__)
    bpy.types.INFO_MT_file_import.remove(ImportPmd.menu_func)
    bpy.types.INFO_MT_file_import.remove(ImportPmx.menu_func)
    bpy.types.INFO_MT_file_import.remove(ImportMqo.menu_func)
    bpy.types.INFO_MT_file_export.remove(ExportPmd.menu_func)
    bpy.types.INFO_MT_file_export.remove(ExportPmx.menu_func)
    bpy.types.INFO_MT_file_export.remove(ExportMqo.menu_func)


if __name__=='__main__':
    register()




# coding: utf-8
import os
import sys
import time
import functools

try:
    import bpy
    import mathutils
except:
    pass

FS_ENCODING=sys.getfilesystemencoding()
if os.path.exists(os.path.dirname(sys.argv[0])+"/utf8"):
    INTERNAL_ENCODING='utf-8'
else:
    INTERNAL_ENCODING=FS_ENCODING

def register():
    pass

def unregister():
    pass

SCENE=None
def initialize(name, scene):
    global SCENE
    SCENE=scene
    progress_start(name)

def finalize():
    scene.update(SCENE)
    progress_finish()

def message(msg):
    print(msg)

def enterEditMode():
    bpy.ops.object.mode_set(mode='EDIT', toggle=False)

def enterObjectMode():
    bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

def enterPoseMode():
    bpy.ops.object.mode_set(mode='POSE', toggle=False)

def createVector(x, y, z):
    return mathutils.Vector([x, y, z])


class Writer(object):
    '''
    io wrapper
    '''
    def __init__(self, path, encoding):
        self.io=open(path, "wb")
        self.encoding=encoding

    def write(self, s):
        self.io.write(s.encode(self.encoding))

    def flush(self):
        self.io.flush()

    def close(self):
        self.io.close()


progressBar=None
class ProgressBar(object):
    '''
    progress bar wrapper
    '''
    def __init__(self, base):
        print("#### %s ####" % base)
        self.base=base
        self.start=time.time() 
        self.set('<start>', 0)

    def advance(self, message, progress):
        self.progress+=float(progress)
        self._print(message)

    def set(self, message, progress):
        self.progress=float(progress)
        self._print(message)

    def _print(self, message):
        print(message)
        message="%s: %s" % (self.base, message)
        #Blender.Window.DrawProgressBar(self.progress, message)

    def finish(self):
        self.progress=1.0
        message='finished in %.2f sec' % (time.time()-self.start)
        self.set(message, 1.0)

def progress_start(base):
    global progressBar
    print("#### progressBar ####")
    progressBar=ProgressBar(base)

def progress_finish():
    global progressBar
    progressBar.finish()

def progress_print(message, progress=0.05):
    global progressBar
    progressBar.advance(message, progress)

def progress_set(message, progress):
    global progressBar
    progressBar.set(message, progress)


class scene:
    @staticmethod
    def get():
        global SCENE
        return SCENE

    def update(scene):
        scene.update()


class object:
    @staticmethod
    def createEmpty(name):
        global SCENE
        empty=bpy.data.objects.new(name, None)
        SCENE.objects.link(empty)
        return empty

    @staticmethod
    def each():
        for o in SCENE.objects:
            yield o

    @staticmethod
    def makeParent(parent, child):
        child.parent=parent

    @staticmethod
    def duplicate(o):
        global SCENE
        bpy.ops.object.select_all(action='DESELECT')
        o.select=True
        SCENE.objects.active=o
        bpy.ops.object.duplicate()
        dumy=SCENE.objects.active
        #bpy.ops.object.rotation_apply()
        #bpy.ops.object.scale_apply()
        #bpy.ops.object.location_apply()
        return dumy.data, dumy

    @staticmethod
    def delete(o):
        global SCENE
        SCENE.objects.unlink(o)

    @staticmethod
    def getData(o):
        return o.data

    @staticmethod
    def select(o):
        o.select=True

    @staticmethod
    def activate(o):
        global SCENE
        o.select=True 
        SCENE.objects.active=o

    @staticmethod
    def getActive():
        global SCENE 
        return SCENE.objects.active

    @staticmethod
    def deselectAll():
        bpy.ops.object.select_all(action='DESELECT')

    @staticmethod
    def setLayerMask(object, layers):
        layer=[]
        for i in range(20):
            try:
                layer.append(True if layers[i]!=0 else False)
            except IndexError:
                layer.append(False)
        object.layers=layer

    @staticmethod
    def isVisible(o):
        return False if o.hide else True

    @staticmethod
    def getShapeKeys(o):
        return o.data.shape_keys.key_blocks

    @staticmethod
    def addShapeKey(o, name):
        try:
            return o.shape_key_add(name)
        except:
            return o.add_shape_key(name)

    @staticmethod
    def hasShapeKey(o):
        return o.data.shape_keys

    @staticmethod
    def pinShape(o, enable):
        o.show_only_shape_key=enable

    @staticmethod
    def setActivateShapeKey(o, index):
        o.active_shape_key_index=index

    @staticmethod
    def getPose(o):
        return o.pose

    @staticmethod
    def getVertexGroup(o, name):
        indices=[]
        for i, v in enumerate(o.data.vertices):
            for g in v.groups:
                if o.vertex_groups[g.group].name==name:
                    indices.append(i)
        return indices

    @staticmethod
    def getVertexGroupNames(o):
        for g in o.vertex_groups:
            yield g.name

    @staticmethod
    def addVertexGroup(o, name):
        o.vertex_groups.new(name)

    @staticmethod
    def assignVertexGroup(o, name, index, weight):
        if name not in o.vertex_groups:
            o.vertex_groups.new(name)
        o.vertex_groups[name].add([index], weight, 'ADD')

    @staticmethod
    def createBoneGroup(o, name, color_set='DEFAULT'):
        # create group
        object.activate(o)
        enterPoseMode()
        bpy.ops.pose.group_add()
        # set name
        pose=object.getPose(o)
        g=pose.bone_groups.active
        g.name=name
        g.color_set=color_set
        return g

    @staticmethod
    def boneGroups(o):
        return object.getPose(o).bone_groups


class modifier:
    @staticmethod
    def addMirror(mesh_object):
        return mesh_object.modifiers.new("Modifier", "MIRROR")

    @staticmethod
    def addArmature(mesh_object, armature_object):
        mod=mesh_object.modifiers.new("Modifier", "ARMATURE")
        mod.object = armature_object
        mod.use_bone_envelopes=False

    @staticmethod
    def hasType(mesh_object, type_name):
        for mod in mesh_object.modifiers:
                if mod.type==type_name.upper():
                    return True

    @staticmethod
    def isType(m, type_name):
        return m.type==type_name.upper()

    @staticmethod
    def getArmatureObject(m):
        return m.object


class shapekey:
    @staticmethod
    def assign(shapeKey, index, pos):
        shapeKey.data[index].co=pos

    @staticmethod
    def getByIndex(b, index):
        return b.data[index].co

    @staticmethod
    def get(b):
        for k in b.data:
            yield k.co


class texture:
    @staticmethod
    def create(path):
        texture=bpy.data.textures.new(os.path.basename(path), 'IMAGE')
        texture.use_mipmap=True
        texture.use_interpolation=True
        texture.use_alpha=True
        try:
            image=bpy.data.images.load(path)
        except RuntimeError:
            print('fail to create:', path)
            image=bpy.data.images.new('Image', width=16, height=16)
        texture.image=image
        return texture, image

    @staticmethod
    def getPath(t):
        if  t.type=="IMAGE":
            image=t.image
            if image:
                return image.filepath


class material:
    @staticmethod
    def create(name):
        return bpy.data.materials.new(name)

    @staticmethod
    def get(material_name):
        return bpy.data.materials[material_name]

    @staticmethod
    def addTexture(material, texture, enable=True):
        # search free slot
        index=None
        for i, slot in enumerate(material.texture_slots):
            if not slot:
                index=i
                break
        if index==None:
            return
        #
        #material.add_texture(texture, "UV", "COLOR")
        #slot=material.texture_slots.add()
        slot=material.texture_slots.create(index)
        slot.texture=texture
        slot.texture_coords='UV'
        slot.blend_type='MULTIPLY'
        slot.use_map_alpha=True
        slot.use=enable
        return index

    @staticmethod
    def getTexture(m, index):
        return m.texture_slots[index].texture

    @staticmethod
    def hasTexture(m):
        return m.texture_slots[0]

    @staticmethod
    def setUseTexture(m, index, enable):
        m.use_textures[index]=enable

    @staticmethod
    def eachTexturePath(m):
        for slot in m.texture_slots:
            if slot and slot.texture:
                texture=slot.texture
                if  texture.type=="IMAGE":
                    image=texture.image
                    if not image:
                        continue
                    yield image.filepath

    @staticmethod
    def eachEnalbeTexturePath(m):
        for i, slot in enumerate(m.texture_slots):
            if m.use_textures[i] and slot and slot.texture:
                texture=slot.texture
                if  texture.type=="IMAGE":
                    image=texture.image
                    if not image:
                        continue
                    yield image.filepath


class mesh:
    @staticmethod
    def create(name):
        global SCENE
        mesh=bpy.data.meshes.new("Mesh")
        mesh_object= bpy.data.objects.new(name, mesh)
        SCENE.objects.link(mesh_object)
        return mesh, mesh_object

    @staticmethod
    def addGeometry(mesh, vertices, faces):
        mesh.from_pydata(vertices, [], faces)
        """
        mesh.add_geometry(len(vertices), 0, len(faces))
        # add vertex
        unpackedVertices=[]
        for v in vertices:
            unpackedVertices.extend(v)
        mesh.vertices.foreach_set("co", unpackedVertices)
        # add face
        unpackedFaces = []
        for face in faces:
            if len(face) == 4:
                if face[3] == 0:
                    # rotate indices if the 4th is 0
                    face = [face[3], face[0], face[1], face[2]]
            elif len(face) == 3:
                if face[2] == 0:
                    # rotate indices if the 3rd is 0
                    face = [face[2], face[0], face[1], 0]
                else:
                    face.append(0)
            unpackedFaces.extend(face)
        mesh.faces.foreach_set("verts_raw", unpackedFaces)
        """
        assert(len(vertices)==len(mesh.vertices))
        assert(len(faces)==len(mesh.faces))

    @staticmethod
    def hasUV(mesh):
        return len(mesh.uv_textures)>0

    @staticmethod
    def useVertexUV(mesh):
        pass

    @staticmethod
    def addUV(mesh):
        mesh.uv_textures.new()

    @staticmethod
    def hasFaceUV(mesh, i, face):
        active_uv_texture=None
        for t in mesh.uv_textures:
            if t.active:
                active_uv_texture=t
                break
        return active_uv_texture and active_uv_texture.data[i]

    @staticmethod
    def getFaceUV(mesh, i, faces, count=3):
        active_uv_texture=None
        for t in mesh.uv_textures:
            if t.active:
                active_uv_texture=t
                break
        if active_uv_texture and active_uv_texture.data[i]:
            uvFace=active_uv_texture.data[i]
            if count==3:
                return (uvFace.uv1, uvFace.uv2, uvFace.uv3)
            elif count==4:
                return (uvFace.uv1, uvFace.uv2, uvFace.uv3, uvFace.uv4)
            else:
                print(count)
                assert(False)
        else:
            return ((0, 0), (0, 0), (0, 0), (0, 0))

    @staticmethod
    def setFaceUV(m, i, face, uv_array, image):
        uv_face=m.uv_textures[0].data[i]
        uv_face.uv=uv_array
        if image:
            uv_face.image=image
            #uv_face.use_image=True

    @staticmethod
    def vertsDelete(m, remove_vertices):
        enterEditMode()
        bpy.ops.mesh.select_all(action='DESELECT')
        enterObjectMode()

        for i in remove_vertices:
            m.vertices[i].select=True

        enterEditMode()
        bpy.ops.mesh.delete(type='VERT')
        enterObjectMode()

    @staticmethod
    def setSmooth(m, smoothing):
        m.auto_smooth_angle=int(smoothing)
        m.use_auto_smooth=True

    @staticmethod
    def recalcNormals(mesh_object):
        bpy.ops.object.select_all(action='DESELECT')
        object.activate(mesh_object)
        enterEditMode()
        bpy.ops.mesh.normals_make_consistent()
        enterObjectMode()

    @staticmethod
    def flipNormals(m):
        m.flipNormals()

    @staticmethod
    def addMaterial(m, material):
        m.materials.append(material)

    @staticmethod
    def getMaterial(m, index):
        return m.materials[index]


class vertex:
    @staticmethod
    def setNormal(v, normal):
        v.normal=mathutils.Vector(normal)

    @staticmethod
    def getNormal(v):
        return v.normal

    @staticmethod
    def setUv(v, uv):
        # sticky ?
        pass


class face:
    @staticmethod
    def getVertexCount(face):
        return len(face.vertices)

    @staticmethod
    def getVertices(face):
        return face.vertices[:]

    @staticmethod
    def getIndices(face, count=3):
        if count==3:
            return [face.vertices[0], face.vertices[1], face.vertices[2]]
        elif count==4:
            return [face.vertices[0], face.vertices[1], face.vertices[2], face.vertices[3]]
        else:
            assert(False)

    @staticmethod
    def setMaterial(face, material_index):
        face.material_index=material_index

    @staticmethod
    def getMaterialIndex(face):
        return face.material_index

    @staticmethod
    def setNormal(face, normal):
        face.normal=normal

    @staticmethod
    def getNormal(face):
        return face.normal

    @staticmethod
    def setSmooth(face, isSmooth):
        face.use_smooth=True if isSmooth else False


class armature:
    @staticmethod
    def create():
        global SCENE
        armature = bpy.data.armatures.new('Armature')
        armature_object=bpy.data.objects.new('Armature', armature)
        SCENE.objects.link(armature_object)

        armature_object.show_x_ray=True
        armature.show_names=True
        #armature.draw_type='OCTAHEDRAL'
        armature.draw_type='STICK'
        armature.use_deform_envelopes=False
        armature.use_deform_vertex_groups=True
        armature.use_mirror_x=True

        return armature, armature_object

    @staticmethod
    def makeEditable(armature_object):
        global SCENE
        # select only armature object and set edit mode
        SCENE.objects.active=armature_object
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
        bpy.ops.object.mode_set(mode='EDIT', toggle=False)

    @staticmethod
    def createIkConstraint(armature_object, p_bone, effector_name, 
            chain, weight, iterations):
        constraint = p_bone.constraints.new('IK')
        constraint.chain_count=len(chain)
        constraint.target=armature_object
        constraint.subtarget=effector_name
        constraint.use_tail=False
        # ToDo
        # not used. place folder when export.
        #constraint.weight=weight
        #constraint.iterations=iterations * 10
        return constraint

    @staticmethod
    def createBone(armature, name):
        return armature.edit_bones.new(name)

    @staticmethod
    def update(armature):
        pass


class bone:
    @staticmethod
    def setConnected(bone):
        bone.use_connect=True

    @staticmethod
    def isConnected(bone):
        return bone.use_connect

    @staticmethod
    def setLayerMask(bone, layers):
        layer=[]
        for i in range(32):
            try:
                layer.append(True if layers[i]!=0 else False)
            except IndexError:
                layer.append(False)
        bone.layers=layer

    @staticmethod
    def getHeadLocal(b):
        return b.head_local[0:3]

    @staticmethod
    def getTailLocal(b):
        return b.tail_local[0:3]


class constraint:
    @staticmethod
    def ikChainLen(c):
        return c.chain_count

    @staticmethod
    def ikTarget(c):
        return c.subtarget

    @staticmethod
    def ikItration(c):
        return c.iterations

    @staticmethod
    def ikRotationWeight(c):
        return c.weight

    @staticmethod
    def isIKSolver(c):
        return c.type=='IK'

MMD_SHAPE_GROUP_NAME='_MMD_SHAPE'
MMD_MB_NAME='mb_name'
MMD_MB_COMMENT='mb_comment'
MMD_COMMENT='comment'
BASE_SHAPE_NAME='Basis'
RIGID_NAME='rigid_name'
RIGID_SHAPE_TYPE='rigid_shape_type'
RIGID_PROCESS_TYPE='rigid_process_type'
RIGID_BONE_NAME='rigid_bone_name'
RIGID_GROUP='ribid_group'
RIGID_INTERSECTION_GROUP='rigid_intersection_group'
RIGID_WEIGHT='rigid_weight'
RIGID_LINEAR_DAMPING='rigid_linear_damping'
RIGID_ANGULAR_DAMPING='rigid_angular_damping'
RIGID_RESTITUTION='rigid_restitution'
RIGID_FRICTION='rigid_friction'
CONSTRAINT_NAME='const_name'
CONSTRAINT_A='const_a'
CONSTRAINT_B='const_b'
CONSTRAINT_POS_MIN='const_pos_min'
CONSTRAINT_POS_MAX='const_pos_max'
CONSTRAINT_ROT_MIN='const_rot_min'
CONSTRAINT_ROT_MAX='const_rot_max'
CONSTRAINT_SPRING_POS='const_spring_pos'
CONSTRAINT_SPRING_ROT='const_spring_rot'
TOON_TEXTURE_OBJECT='ToonTextures'




#!/usr/bin/python
# coding: utf-8

import math
from OpenGL.GL import *
from OpenGL.GLU import *

from . import baseview

class RokuroView(baseview.BaseView):
    def __init__(self):
        super(RokuroView, self).__init__()
        self.w=1
        self.h=1
        self.head=0
        self.pitch=0
        self.SHIFT_FACTOR=0.001
        self.distance=100
        self.shiftX=0
        self.shiftY=0
        self.aspect=1
        self.n=1
        self.f=10000

    def __str__(self):
        return '<RokuroView shiftX: %f, shiftY: %f, distance: %f>' % (
                self.shiftX, self.shiftY, self.distance)
 
    def onResize(self, w=None, h=None):
        super(RokuroView, self).onResize(w, h)
        self.aspect=float(self.w)/float(self.h)

    def dolly(self, d):
        if d>0:
            self.distance*=1.1
        elif d<0:
            self.distance*=0.9

    def shift(self, dx, dy):
        self.shiftX+=dx * self.distance * self.SHIFT_FACTOR
        self.shiftY+=dy * self.distance * self.SHIFT_FACTOR

    def rotate(self, head, pitch):
        self.head+=head
        self.pitch+=pitch

    def updateProjection(self):
        gluPerspective(30, self.aspect, self.n, self.f)

    def updateView(self):
        glTranslate(self.shiftX, self.shiftY, -self.distance)
        glRotate(self.head, 0, 1, 0)
        glRotate(self.pitch, 1, 0, 0)

    def onMotion(self, x, y):
        redraw=False
        if self.isLeftDown:
            self.dolly(y-self.y)
            redraw=True
        if self.isMiddelDown:
            self.shift(x-self.x, self.y-y)
            redraw=True
        if self.isRightDown:
            self.rotate(x-self.x, y-self.y)
            redraw=True
        self.x=x
        self.y=y
        return redraw

    def onWheel(self, d):
        if d!=0:
            self.dolly(d)
            return True

    def look_bb(self, min_v, max_v):
        w=max_v[0]-min_v[0]
        h=max_v[1]-min_v[1]
        long_side=max(w, h)
        def deglee_to_radian(deglee):
            return math.pi*deglee/180.0
        d=long_side/math.tan(deglee_to_radian(30)) * 1.5
        self.distance=min_v[2]+d
        cx=min_v[0]+max_v[0]
        cy=min_v[1]+max_v[1]
        self.shiftX=-cx/2.0
        self.shiftY=-cy/2.0
        print(self)




# coding: utf-8
"""
Blenderのメッシュをワンスキンメッシュ化する
"""
from .. import bl
from . import oneskinmesh
from . import bonebuilder


class ObjectNode(object):
    '''
    Objectの木構造構築
    '''
    __slots__=['o', 'children']
    def __init__(self, o):
        self.o=o
        self.children=[]


class Exporter(object):
    '''
    Blenderから情報収集する
    '''
    __slots__=[
            'oneSkinMesh',
            'skeleton',
            'root',
            ]
    def setup(self):
        # scene内のオブジェクトの木構造を構築する
        object_node_map={}
        for o in bl.object.each():
            object_node_map[o]=ObjectNode(o)
        for o in bl.object.each():
            node=object_node_map[o]
            if node.o.parent:
                object_node_map[node.o.parent].children.append(node)
        self.root=object_node_map[bl.object.getActive()]

        # ワンスキンメッシュを作る
        self.oneSkinMesh=oneskinmesh.OneSkinMesh()
        self.oneSkinMesh.build(self.root)
        bl.message(self.oneSkinMesh)
        if len(self.oneSkinMesh.morphList)==0:
            # create emtpy skin
            self.oneSkinMesh.createEmptyBasicSkin()

        # skeleton
        self.skeleton=bonebuilder.BoneBuilder()
        self.skeleton.build(self.oneSkinMesh.armatureObj)




#!/usr/bin/env python
# coding: utf-8

import time
import os
import pymeshio.mqo.reader
import opengl.material
import opengl.texture
import opengl.vertexarraymap


def build(path):
    t=time.time()
    model=pymeshio.mqo.reader.read_from_file(path)
    if not model:
        return
    print(time.time()-t, "sec")
    # build
    basedir=os.path.dirname(path)
    vertexArrayMap=opengl.vertexarraymap.VertexArrayMapWithUV()
    for m in model.materials:
        material=opengl.material.MQOMaterial()
        material.rgba=(m.color.r, m.color.g, m.color.b, m.color.a)
        if m.tex:
            texturepath=os.path.join(basedir, m.tex.decode('cp932'))
            material.texture=opengl.texture.Texture(texturepath)
        vertexArrayMap.addMaterial(material)

    for o in model.objects:
        # skip mikoto objects
        if o.name.startswith(b"anchor"):
            continue
        if o.name.startswith(b"bone:"):
            continue
        if o.name.startswith(b"MCS:"):
            continue

        for f in o.faces:
            if f.index_count==3:
                vertexArrayMap.addTriangle(
                        f.material_index,
                        o.vertices[f.indices[0]],
                        o.vertices[f.indices[1]],
                        o.vertices[f.indices[2]],
                        f.uv[0], f.uv[1], f.uv[2]
                        )
            elif f.index_count==4:
                # triangle 1
                vertexArrayMap.addTriangle(
                        f.material_index,
                        o.vertices[f.indices[0]],
                        o.vertices[f.indices[1]],
                        o.vertices[f.indices[2]],
                        f.uv[0], f.uv[1], f.uv[2]
                        )
                # triangle 2
                vertexArrayMap.addTriangle(
                        f.material_index,
                        o.vertices[f.indices[2]],
                        o.vertices[f.indices[3]],
                        o.vertices[f.indices[0]],
                        f.uv[2], f.uv[3], f.uv[0]
                        )

    vertexArrayMap.optimize()
    return vertexArrayMap




# coding: utf-8
"""
"""


import sys
import os
import io
from .pmd import reader
from .pmx import writer
from . import converter


def pmd_to_pmx():
    if len(sys.argv)<3:
        print("usage: %s {input pmd_file} {out pmx_file}" % os.path.basename(sys.argv[0]))
        sys.exit()
    pmd=reader.read_from_file(sys.argv[1])
    pmx=converter.pmd_to_pmx(pmd)
    writer.write(io.open(sys.argv[2], "wb"), pmx)

def pmd_diff():
    if len(sys.argv)<3:
        print("sage: %s {pmd_file} {pmd_file}" % os.path.basename(sys.argv[0]))
        sys.exit()
    lhs=reader.read_from_file(sys.argv[1])
    rhs=reader.read_from_file(sys.argv[2])
    lhs.diff(rhs)

def pmd_validator():
    if len(sys.argv)==1:
        print("sage: %s {input pmd_file}" % os.path.basename(sys.argv[0]))
        sys.exit()
    pmd=reader.read_from_file(sys.argv[1])
    print "%s(%s)" % (pmd.name.decode('cp932'), pmd.english_name.decode('cp932'))
    print "vertices: %d" % len(pmd.vertices)
    print "indices: %d" % len(pmd.indices)
    print "materials: %d" % len(pmd.materials)
    print "bones: %d" % len(pmd.bones)
    print "ik_list: %d" % len(pmd.ik_list)
    print "morphs: %d" % len(pmd.morphs)
    print "morph_indices: %d" % len(pmd.morph_indices)
    print "bone_group_list: %d" % len(pmd.bone_group_list)
    print "bone_display_list: %d" % len(pmd.bone_group_list)
    print "toon_textures: %d" % len(pmd.toon_textures)
    print "rigidbodies: %d" % len(pmd.rigidbodies)
    print "joints: %d" % len(pmd.joints)




#!BPY
# coding: utf-8
""" 
Name: 'Metasequoia(.mqo)...'
Blender: 245
Group: 'Import'
Tooltip: 'Import from Metasequoia file format (.mqo)'
"""
__author__=['ousttrue']
__url__ = ["http://gunload.web.fc2.com/blender/"]
__bpydoc__= '''\

MQO Importer

This script imports a mqo into Blender for editing.

20080123: update.
20091125: modify for linux.
20100310: rewrite.
20100311: create armature from mikoto bone.
20100505: C extension.
20100606: integrate 2.4 and 2.5.
20100619: fix multibyte object name.
20100626: refactoring.
20100724: update for Blender2.53.
20100731: add full python module.
20101005: update for Blender2.54.
20101228: update for Blender2.55.
20110429: update for Blender2.57b.
20110918: update for Blender2.59.
20111002: update for pymeshio-2.1.0
'''

bl_addon_info = {
        'category': 'Import/Export',
        'name': 'Import: Metasequioa Model Format (.mqo)',
        'author': 'ousttrue',
        'version': (2, 0),
        'blender': (2, 5, 3),
        'location': 'File > Import',
        'description': 'Import from the Metasequioa Model Format (.mqo)',
        'warning': '', # used for warning icon and text in addons panel
        'wiki_url': 'http://sourceforge.jp/projects/meshio/wiki/FrontPage',
        'tracker_url': 'http://sourceforge.jp/ticket/newticket.php?group_id=5081',
        }

import os
import sys
from .pymeshio.mqo import reader

# for 2.5
import bpy

# wrapper
from . import bl

def createMqoMaterial(m):
    material = bpy.data.materials.new(m.name.decode("cp932"))
    # shader
    if m.shader==1:
        material.diffuse_shader='FRESNEL'
    else:
        material.diffuse_shader='LAMBERT'
    # diffuse
    material.diffuse_color=[m.color.r, m.color.g, m.color.b]
    material.diffuse_intensity=m.diffuse
    material.alpha=m.color.a
    # other
    material.ambient = m.ambient
    #material.specular = m.specular
    material.emit=m.emit
    material.use_shadeless=True
    return material


def has_mikoto(mqo):
    return False


def __createMaterials(mqo, directory):
    """
    create blender materials and renturn material list.
    """
    materials = []
    textureMap={}
    imageMap={}
    if len(mqo.materials)>0:
        for material_index, m in enumerate(mqo.materials):
            # material
            material=createMqoMaterial(m)
            materials.append(material)
            # texture
            texture_name=m.tex.decode("cp932")
            if texture_name!=b'':
                if texture_name in textureMap:
                    texture=textureMap[texture_name]
                else:
                    # load texture image
                    if os.path.isabs(texture_name):
                        # absolute
                        path = texture_name
                    else:
                        # relative
                        path = os.path.join(directory, texture_name)
                    # texture
                    path=path.replace("\\", "/")
                    if os.path.exists(path):
                        print("create texture:", path)
                        texture, image=bl.texture.create(path)
                        textureMap[texture_name]=texture
                        imageMap[material_index]=image
                    else:
                        print("%s not exits" % path)
                        continue
                bl.material.addTexture(material, texture)
    else:
        # default material
        pass
    return materials, imageMap


def __createObjects(mqo, root, materials, imageMap, scale):
    """
    create blender mesh objects.
    """
    # tree stack
    stack=[root]    
    objects=[]
    for o in mqo.objects:
        mesh, mesh_object=bl.mesh.create(o.name.decode("cp932"))

        # add hierarchy
        stack_depth=len(stack)-1
        #print(o.depth, stack_depth)
        if o.depth<stack_depth:
            for i in range(stack_depth-o.depth):
                stack.pop()
        bl.object.makeParent(stack[-1], mesh_object)
        stack.append(mesh_object)

        obj_name=o.name.decode("cp932")
        if obj_name.startswith('sdef'):
            objects.append(mesh_object)
        elif obj_name.startswith('anchor'):
            bl.object.setLayerMask(mesh_object, [0, 1])
        elif obj_name.startswith('bone'):
            bl.object.setLayerMask(mesh_object, [0, 1])

        # geometry
        vertices=[(v.x * scale, -v.z * scale, v.y * scale) for v in o.vertices]
        faces=[]
        materialMap={}
        for f in o.faces:
            face_indices=[]
            # flip face
            for i in reversed(range(f.index_count)):
                face_indices.append(f.getIndex(i))
            faces.append(face_indices)
            materialMap[f.material_index]=True
        bl.mesh.addGeometry(mesh, vertices, faces)

        # blender limits 16 materials per mesh
        for i, material_index in enumerate(materialMap.keys()):
            if i>=16:
                # split a mesh ?
                print("over 16 materials!")
                break
            bl.mesh.addMaterial(mesh, materials[material_index])
            materialMap[material_index]=i
 
        # set face params
        assert(len(o.faces)==len(mesh.faces))
        bl.mesh.addUV(mesh)
        for i, (f, face) in enumerate(zip(o.faces, mesh.faces)):
            uv_array=[]
            # ToDo FIX
            # flip face
            for j in reversed(range(f.index_count)):
                uv_array.append((f.getUV(j).x, 1.0-f.getUV(j).y))
            bl.mesh.setFaceUV(mesh, i, face, uv_array, 
                    imageMap.get(f.material_index, None))
            if f.material_index in materialMap:
                bl.face.setMaterial(face, materialMap[f.material_index])
            bl.face.setSmooth(face, True)

        # mirror modifier
        if o.mirror:
            bl.modifier.addMirror(mesh_object)

        # set smoothing
        bl.mesh.setSmooth(mesh, o.smoothing)

        # calc normal
        bl.mesh.recalcNormals(mesh_object)

    return objects


###############################################################################
# for mqo mikoto bone.
###############################################################################
class MikotoBone(object):
    __slots__=[
            'name',
            'iHead', 'iTail', 'iUp',
            'vHead', 'vTail', 'vUp',
            'parent', 'isFloating',
            'children',
            ]
    def __init__(self, face=None, vertices=None, materials=None):
        self.parent=None
        self.isFloating=False
        self.children=[]
        if not face:
            self.name='root'
            return

        self.name=materials[face.material_index].name.encode('utf-8')

        i0=face.getIndex(0)
        i1=face.getIndex(1)
        i2=face.getIndex(2)
        v0=vertices[i0]
        v1=vertices[i1]
        v2=vertices[i2]
        e01=v1-v0
        e12=v2-v1
        e20=v0-v2
        sqNorm0=e01.getSqNorm()
        sqNorm1=e12.getSqNorm()
        sqNorm2=e20.getSqNorm()
        if sqNorm0>sqNorm1:
            if sqNorm1>sqNorm2:
                # e01 > e12 > e20
                self.iHead=i2
                self.iTail=i1
                self.iUp=i0
            else:
                if sqNorm0>sqNorm2:
                    # e01 > e20 > e12
                    self.iHead=i2
                    self.iTail=i0
                    self.iUp=i1
                else:
                    # e20 > e01 > e12
                    self.iHead=i1
                    self.iTail=i0
                    self.iUp=i2
        else:
            # 0 < 1
            if sqNorm1<sqNorm2:
                # e20 > e12 > e01
                self.iHead=i1
                self.iTail=i2
                self.iUp=i0
            else:
                if sqNorm0<sqNorm2:
                    # e12 > e20 > e01
                    self.iHead=i0
                    self.iTail=i2
                    self.iUp=i1
                else:
                    # e12 > e01 > e20
                    self.iHead=i0
                    self.iTail=i1
                    self.iUp=i2
        self.vHead=vertices[self.iHead]
        self.vTail=vertices[self.iTail]
        self.vUp=vertices[self.iUp]

        if self.name.endswith('[]'):
            basename=self.name[0:-2]
            # expand LR name
            if self.vTail.x>0:
                self.name="%s_L" % basename
            else:
                self.name="%s_R" % basename


    def setParent(self, parent, floating=False):
        if floating:
            self.isFloating=True
        self.parent=parent
        parent.children.append(self)

    def printTree(self, indent=''):
        print("%s%s" % (indent, self.name))
        for child in self.children:
            child.printTree(indent+'  ')


def build_armature(armature, mikotoBone, parent=None):
    """
    create a armature bone.
    """
    bone = Armature.Editbone()
    bone.name = mikotoBone.name.encode('utf-8')
    armature.bones[bone.name] = bone

    bone.head = Mathutils.Vector(*mikotoBone.vHead.to_a())
    bone.tail = Mathutils.Vector(*mikotoBone.vTail.to_a())
    if parent:
        bone.parent=parent
        if mikotoBone.isFloating:
            pass
        else:
            bone.options=[Armature.CONNECTED]

    for child in mikotoBone.children:
        build_armature(armature, child, bone)


def create_armature(mqo):
    """
    create armature
    """
    boneObject=None
    for o in mqo.objects:
        if o.name.startswith('bone'):
            boneObject=o
            break
    if not boneObject:
        return

    tailMap={}
    for f in boneObject.faces:
        if f.index_count!=3:
            print("invalid index_count: %d" % f.index_count)
            continue
        b=MikotoBone(f, boneObject.vertices, mqo.materials)
        tailMap[b.iTail]=b

    #################### 
    # build mikoto bone tree
    #################### 
    mikotoRoot=MikotoBone()

    for b in tailMap.values():
        # each bone has unique parent or is root bone.
        if b.iHead in tailMap:
            b.setParent(tailMap[b.iHead])
        else: 
            isFloating=False
            for e in boneObject.edges:
                if  b.iHead==e.indices[0]:
                    # floating bone
                    if e.indices[1] in tailMap:
                        b.setParent(tailMap[e.indices[1]], True)
                        isFloating=True
                        break
                elif b.iHead==e.indices[1]:
                    # floating bone
                    if e.indices[0] in tailMap:
                        b.setParent(tailMap[e.indices[0]], True)
                        isFloating=True
                        break
            if isFloating:
                continue

            # no parent bone
            b.setParent(mikotoRoot, True)

    if len(mikotoRoot.children)==0:
        print("no root bone")
        return

    if len(mikotoRoot.children)==1:
        # single root
        mikotoRoot=mikotoRoot.children[0]
        mikotoRoot.parent=None
    else:
        mikotoRoot.vHead=Vector3(0, 10, 0)
        mikotoRoot.vTail=Vector3(0, 0, 0)

    #################### 
    # create armature
    #################### 
    armature = Armature.New()
    # link to object
    armature_object = scene.objects.new(armature)
    # create action
    act = Armature.NLA.NewAction()
    act.setActive(armature_object)
    # set XRAY
    armature_object.drawMode |= Object.DrawModes.XRAY
    # armature settings
    armature.drawType = Armature.OCTAHEDRON
    armature.envelopes = False
    armature.vertexGroups = True
    armature.mirrorEdit = True
    armature.drawNames=True

    # edit bones
    armature.makeEditable()
    build_armature(armature, mikotoRoot)
    armature.update()

    return armature_object
        

class TrianglePlane(object):
    """
    mikoto方式ボーンのアンカーウェイト計算用。
    (不完全)
    """
    __slots__=['normal', 
            'v0', 'v1', 'v2',
            ]
    def __init__(self, v0, v1, v2):
        self.v0=v0
        self.v1=v1
        self.v2=v2

    def isInsideXY(self, p):
        v0=Vector2(self.v0.x, self.v0.y)
        v1=Vector2(self.v1.x, self.v1.y)
        v2=Vector2(self.v2.x, self.v2.y)
        e01=v1-v0
        e12=v2-v1
        e20=v0-v2
        c0=Vector2.cross(e01, p-v0)
        c1=Vector2.cross(e12, p-v1)
        c2=Vector2.cross(e20, p-v2)
        if c0>=0 and c1>=0 and c2>=0:
            return True
        if c0<=0 and c1<=0 and c2<=0:
            return True

    def isInsideYZ(self, p):
        v0=Vector2(self.v0.y, self.v0.z)
        v1=Vector2(self.v1.y, self.v1.z)
        v2=Vector2(self.v2.y, self.v2.z)
        e01=v1-v0
        e12=v2-v1
        e20=v0-v2
        c0=Vector2.cross(e01, p-v0)
        c1=Vector2.cross(e12, p-v1)
        c2=Vector2.cross(e20, p-v2)
        if c0>=0 and c1>=0 and c2>=0:
            return True
        if c0<=0 and c1<=0 and c2<=0:
            return True

    def isInsideZX(self, p):
        v0=Vector2(self.v0.z, self.v0.x)
        v1=Vector2(self.v1.z, self.v1.x)
        v2=Vector2(self.v2.z, self.v2.x)
        e01=v1-v0
        e12=v2-v1
        e20=v0-v2
        c0=Vector2.cross(e01, p-v0)
        c1=Vector2.cross(e12, p-v1)
        c2=Vector2.cross(e20, p-v2)
        if c0>=0 and c1>=0 and c2>=0:
            return True
        if c0<=0 and c1<=0 and c2<=0:
            return True


class MikotoAnchor(object):
    """
    mikoto方式スケルトンのアンカー。
    """
    __slots__=[
            "triangles", "bbox",
            ]
    def __init__(self):
        self.triangles=[]
        self.bbox=None

    def push(self, face, vertices):
        if face.index_count==3:
            self.triangles.append(TrianglePlane(
                vertices[face.indices[0]],
                vertices[face.indices[1]],
                vertices[face.indices[2]]
                ))
        elif face.index_count==4:
            self.triangles.append(TrianglePlane(
                vertices[face.indices[0]],
                vertices[face.indices[1]],
                vertices[face.indices[2]]
                ))
            self.triangles.append(TrianglePlane(
                vertices[face.indices[2]],
                vertices[face.indices[3]],
                vertices[face.indices[0]]
                ))
        # bounding box
        if not self.bbox:
            self.bbox=BoundingBox(vertices[face.indices[0]])
        for i in face.indices:
            self.bbox.expand(vertices[i])


    def calcWeight(self, v):
        if not self.bbox.isInside(v):
            return 0

        if self.anyXY(v.x, v.y) and self.anyYZ(v.y, v.z) and self.anyZX(v.z, v.x):
            return 1.0
        else:
            return 0
        
    def anyXY(self, x, y):
        for t in self.triangles:
            if t.isInsideXY(Vector2(x, y)):
                return True
        return False

    def anyYZ(self, y, z):
        for t in self.triangles:
            if t.isInsideYZ(Vector2(y, z)):
                return True
        return False

    def anyZX(self, z, x):
        for t in self.triangles:
            if t.isInsideZX(Vector2(z, x)):
                return True
        return False


def create_bone_weight(scene, mqo, armature_object, objects):
    """
    create mikoto bone weight.
    """
    anchorMap={}
    # setup mikoto anchors
    for o in mqo.objects:
        if o.name.startswith("anchor"):
            for f in o.faces:
                name=mqo.materials[f.material_index].name
                if name.endswith('[]'):
                    basename=name[0:-2]
                    v=o.vertices[f.indices[0]]
                    if(v.x>0):
                        # L
                        name_L=basename+'_L'
                        if not name_L in anchorMap:
                            anchorMap[name_L]=MikotoAnchor()
                        anchorMap[name_L].push(f, o.vertices)
                    elif(v.x<0):
                        # R
                        name_R=basename+'_R'
                        if not name_R in anchorMap:
                            anchorMap[name_R]=MikotoAnchor()
                        anchorMap[name_R].push(f, o.vertices)
                    else:
                        print("no side", v)
                else:
                    if not name in anchorMap:
                        anchorMap[name]=MikotoAnchor()
                    anchorMap[name].push(f, o.vertices)

    for o in objects:
        # add armature modifier
        mod=o.modifiers.append(Modifier.Types.ARMATURE)
        mod[Modifier.Settings.OBJECT] = armature_object
        mod[Modifier.Settings.ENVELOPES] = False
        o.makeDisplayList()
        # create vertex group
        mesh=o.getData(mesh=True)
        for name in anchorMap.keys():
            mesh.addVertGroup(name)
        mesh.update()
                 
    # assing vertices to vertex group
    for o in objects:
        mesh=o.getData(mesh=True)
        for i, mvert in enumerate(mesh.verts):
            hasWeight=False
            for name, anchor in anchorMap.items():
                weight=anchor.calcWeight(mvert.co)
                if weight>0:
                    mesh.assignVertsToGroup(
                            name, [i], weight, Mesh.AssignModes.ADD)
                    hasWeight=True
            if not hasWeight:
                # debug orphan vertex
                print('orphan', mvert)
        mesh.update()


def _execute(filepath='', scale=0.1):
    # read mqo model
    model=reader.read_from_file(filepath)
    if not model:
        bl.message("fail to load %s" % filepath)
        return

    # create materials
    materials, imageMap=__createMaterials(model, os.path.dirname(filepath))
    if len(materials)==0:
        materials.append(bl.material.create('default'))

    # create objects
    root=bl.object.createEmpty(os.path.basename(filepath))
    objects=__createObjects(model, root, materials, imageMap, scale)

    if has_mikoto(model):
        # create mikoto bone
        armature_object=create_armature(model)
        if armature_object:
            root.makeParent([armature_object])

            # create bone weight
            create_bone_weight(model, armature_object, objects)




#coding: utf-8
"""
pmd reader
"""
import io
from .. import common
from .. import pmd


class Reader(common.BinaryReader):
    """pmx reader
    """
    def __init__(self, ios, version):
        super(Reader, self).__init__(ios)
        self.version=version

    def read_text(self, size):
        """read cp932 text
        """
        src=self.unpack("%ds" % size, size)
        assert(type(src)==bytes)
        pos = src.find(b"\x00")
        if pos==-1:
            return src
        else:
            return src[:pos]

    def read_vertex(self):
        return pmd.Vertex(
                self.read_vector3(),
                self.read_vector3(),
                self.read_vector2(),
                self.read_uint(2),
                self.read_uint(2),
                self.read_uint(1),
                self.read_uint(1))

    def read_material(self):
        return pmd.Material(
                diffuse_color=self.read_rgb(),
                alpha=self.read_float(),
                specular_factor=self.read_float(),
                specular_color=self.read_rgb(),
                ambient_color=self.read_rgb(),
                toon_index=self.read_int(1),
                edge_flag=self.read_uint(1),
                vertex_count=self.read_uint(4),
                texture_file=self.read_text(20)
                )

    def read_bone(self):
        name=self.read_text(20)
        parent_index=self.read_uint(2)
        tail_index=self.read_uint(2)
        bone=pmd.createBone(name, self.read_uint(1))
        bone.parent_index=parent_index
        bone.tail_index=tail_index
        bone.ik_index = self.read_uint(2)
        bone.pos = self.read_vector3()
        return bone

    def read_ik(self):
        ik=pmd.IK(self.read_uint(2), self.read_uint(2))
        ik.length = self.read_uint(1)
        ik.iterations = self.read_uint(2)
        ik.weight = self.read_float()
        ik.children=[self.read_uint(2) for _ in range(ik.length)]
        return ik

    def read_morph(self):
        morph=pmd.Morph(self.read_text(20))
        morph_size = self.read_uint(4)
        morph.type = self.read_uint(1)
        for j in range(morph_size):
            morph.indices.append(self.read_uint(4))
            morph.pos_list.append(self.read_vector3())
        return morph

    def read_rigidbody(self):
        return pmd.RigidBody(
                name=self.read_text(20), 
                bone_index=self.read_int(2),
                collision_group=self.read_int(1),
                no_collision_group=self.read_int(2),
                shape_type=self.read_uint(1),
                shape_size=self.read_vector3(),
                shape_position=self.read_vector3(),
                shape_rotation=self.read_vector3(),
                mass=self.read_float(),
                linear_damping=self.read_float(),
                angular_damping=self.read_float(),
                restitution=self.read_float(),
                friction=self.read_float(),
                mode=self.read_uint(1)
                )

    def read_joint(self):
        return pmd.Joint(
                name=self.read_text(20),
                rigidbody_index_a=self.read_uint(4),
                rigidbody_index_b=self.read_uint(4),
                position=self.read_vector3(),
                rotation=self.read_vector3(),
                translation_limit_min=self.read_vector3(),
                translation_limit_max=self.read_vector3(),
                rotation_limit_min=self.read_vector3(),
                rotation_limit_max=self.read_vector3(),
                spring_constant_translation=self.read_vector3(),
                spring_constant_rotation=self.read_vector3())



def __read(reader, model):
    # model info
    model.name=reader.read_text(20)
    model.comment=reader.read_text(256) 

    # model data
    model.vertices=[reader.read_vertex()
            for _ in range(reader.read_uint(4))]
    model.indices=[reader.read_uint(2)
            for _ in range(reader.read_uint(4))]
    model.materials=[reader.read_material()
            for _ in range(reader.read_uint(4))]
    model.bones=[reader.read_bone()
            for _ in range(reader.read_uint(2))]
    model.ik_list=[reader.read_ik()
            for _ in range(reader.read_uint(2))]
    model.morphs=[reader.read_morph()
            for _ in range(reader.read_uint(2))]
    model.morph_indices=[reader.read_uint(2)
            for _ in range(reader.read_uint(1))]
    model.bone_group_list=[pmd.BoneGroup(reader.read_text(50))
            for _ in range(reader.read_uint(1))]
    model.bone_display_list=[(reader.read_uint(2), reader.read_uint(1))
            for _i in range(reader.read_uint(4))]

    if reader.is_end():
        # EOF
        return True

    ############################################################
    # extend1: english name
    ############################################################
    if reader.read_uint(1)==1:
        #return True
        model.english_name=reader.read_text(20)
        model.english_comment=reader.read_text(256)
        for bone in model.bones:
            bone.english_name=reader.read_text(20)
        for morph in model.morphs:
            if morph.name==b'base':
                continue
            morph.english_name=reader.read_text(20)
        for g in model.bone_group_list:
            g.english_name=reader.read_text(50)


    ############################################################
    # extend2: toon_textures
    ############################################################
    if reader.is_end():
        # EOF
        return True
    model.toon_textures=[reader.read_text(100)
            for _ in range(10)]

    ############################################################
    # extend2: rigidbodies and joints
    ############################################################
    if reader.is_end():
        # EOF
        return True

    model.rigidbodies=[reader.read_rigidbody()
            for _ in range(reader.read_uint(4))]
    model.joints=[reader.read_joint()
            for _ in range(reader.read_uint(4))]

    return True


def read_from_file(path):
    """
    read from file path, then return the pymeshio.pmd.Model.

    :Parameters:
      path
        file path

    >>> import pymeshio.pmd.reader
    >>> m=pymeshio.pmd.reader.read_from_file('resources/初音ミクVer2.pmd')
    >>> print(m)
    <pmd-2.0 "Miku Hatsune" 12354vertices>

    """
    pmd=read(io.BytesIO(common.readall(path)))
    pmd.path=path
    return pmd


def read(ios):
    """
    read from ios, then return the pymeshio.pmd.Model.

    :Parameters:
      ios
        input stream (in io.IOBase)

    >>> import pymeshio.pmd.reader
    >>> m=pymeshio.pmd.reader.read(io.open('resources/初音ミクVer2.pmd', 'rb'))
    >>> print(m)
    <pmd-2.0 "Miku Hatsune" 12354vertices>

    """
    assert(isinstance(ios, io.IOBase))
    reader=common.BinaryReader(ios)

    # header
    signature=reader.unpack("3s", 3)
    if signature!=b"Pmd":
        raise common.ParseException(
                "invalid signature: {0}".format(signature))
    version=reader.read_float()

    model=pmd.Model(version)
    reader=Reader(reader.ios, version)
    if(__read(reader, model)):
        # check eof
        if not reader.is_end():
            #print("can not reach eof.")
            pass

        # build bone tree
        for i, child in enumerate(model.bones):
            child.index=i
            if child.parent_index==0xFFFF:
                # no parent
                model.no_parent_bones.append(child)
                child.parent=None
            else:
                # has parent
                parent=model.bones[child.parent_index]
                child.parent=parent
                parent.children.append(child)
            # 後位置
            if child.hasChild():
                child.tail=model.bones[child.tail_index].pos

        return model




# coding: utf-8
"""
pmx writer
"""
import io
import struct
from .. import common
from .. import pmx

class Writer(common.BinaryWriter):
    """pmx writer
    """
    def __init__(self, ios,
            text_encoding, extended_uv,
            vertex_index_size, texture_index_size, material_index_size,
            bone_index_size, morph_index_size, rigidbody_index_size):
        super(Writer, self).__init__(ios)
        if text_encoding==0:
            def write_text(unicode):
               utf16=unicode.encode('utf16') 
               self.write_int(len(utf16), 4)
               self.write_bytes(utf16)
            self.write_text=write_text
        elif text_encoding==1:
            def write_text(unicode):
               utf8=unicode.encode('utf8') 
               self.write_int(len(utf8), 4)
               self.write_bytes(utf8)
            self.write_text=write_text
        else:
            raise WriteError(
                    "invalid text_encoding: {0}".format(text_encoding))

        self.write_vertex_index=lambda index: self.write_int(index, vertex_index_size)
        self.write_texture_index=lambda index: self.write_int(index, texture_index_size)
        self.write_material_index=lambda index: self.write_int(index, material_index_size)
        self.write_bone_index=lambda index: self.write_int(index, bone_index_size)
        self.write_morph_index=lambda index: self.write_int(index, morph_index_size)
        self.write_rigidbody_index=lambda index: self.write_int(index, rigidbody_index_size)

    def write_vertices(self, vertices):
        self.write_int(len(vertices), 4)
        for v in vertices:
            self.write_vector3(v.position)
            self.write_vector3(v.normal)
            self.write_vector2(v.uv)
            self.write_deform(v.deform)
            self.write_float(v.edge_factor)

    def write_deform(self, deform):
        if isinstance(deform, pmx.Bdef1):
            self.write_int(0, 1)
            self.write_bone_index(deform.index0)
        elif isinstance(deform, pmx.Bdef2):
            self.write_int(1, 1)
            self.write_bone_index(deform.index0)
            self.write_bone_index(deform.index1)
            self.write_float(deform.weight0)
        elif isinstance(deform, pmx.Bdef4):
            # todo
            raise pymeshio.common.WriteException(
                    "not implemented Bdef4")
        else:
            raise pymeshio.common.WriteException(
                    "unknown deform type: {0}".format(deform.type))

    def write_indices(self, indices):
        self.write_int(len(indices), 4)
        for i in indices:
            self.write_vertex_index(i)

    def write_textures(self, textures):
        self.write_int(len(textures), 4)
        for t in textures:
            self.write_text(t)

    def write_materials(self, materials):
        self.write_int(len(materials), 4)
        for m in materials:
            self.write_text(m.name)
            self.write_text(m.english_name)
            self.write_rgb(m.diffuse_color)
            self.write_float(m.alpha)
            self.write_rgb(m.specular_color)
            self.write_float(m.specular_factor)
            self.write_rgb(m.ambient_color)
            self.write_int(m.flag, 1)
            self.write_rgba(m.edge_color)
            self.write_float(m.edge_size)
            self.write_texture_index(m.texture_index)
            self.write_texture_index(m.sphere_texture_index)
            self.write_int(m.sphere_mode, 1)
            self.write_int(m.toon_sharing_flag, 1)
            if m.toon_sharing_flag==0:
                self.write_texture_index(m.toon_texture_index)
            elif m.toon_sharing_flag==1:
                self.write_int(m.toon_texture_index, 1)
            else:
                raise common.WriteException(
                        "unknown toon_sharing_flag {0}".format(m.toon_sharing_flag))
            self.write_text(m.comment)
            self.write_int(m.vertex_count, 4)

    def write_bones(self, bones):
        self.write_int(len(bones), 4)
        for bone in bones:
            self.write_text(bone.name)
            self.write_text(bone.english_name)
            self.write_vector3(bone.position)
            self.write_bone_index(bone.parent_index)
            self.write_int(bone.layer, 4)
            self.write_int(bone.flag, 2)
            if bone.getConnectionFlag()==0:
                self.write_vector3(bone.tail_position)
            elif bone.getConnectionFlag()==1:
                self.write_bone_index(bone.tail_index)
            else:
                raise pymeshio.common.WriteException(
                        "unknown bone conenction flag: {0}".format(
                            bone.getConnectionFlag()))

            if bone.getRotationFlag()==1 or bone.getTranslationFlag()==1:
                self.write_bone_index(bone.effect_index)
                self.write_float(bone.effect_factor)

            if bone.getFixedAxisFlag()==1:
                self.write_vector3(bone.fixed_axis)

            if bone.getLocalCoordinateFlag()==1:
                self.write_vector3(bone.local_x_vector)
                self.write_vector3(bone.local_z_vector)

            if bone.getExternalParentDeformFlag()==1:
                self.write_int(bone.external_key, 4)

            if bone.getIkFlag()==1:
                self.write_ik(bone.ik)

    def write_ik(self, ik):
        self.write_bone_index(ik.target_index)
        self.write_int(ik.loop, 4)
        self.write_float(ik.limit_radian)
        self.write_int(len(ik.link), 4)
        for l in ik.link:
            self.write_ik_link(l)

    def write_ik_link(self, link):
        self.write_bone_index(link.bone_index)
        self.write_int(link.limit_angle, 1)
        if link.limit_angle==0:
            pass
        elif link.limit_angle==1:
            self.write_vector3(link.limit_min)
            self.write_vector3(link.limit_max)
        else:
            raise pymeshio.common.WriteException(
                    "invalid ik link limit_angle: {0}".format(
                        link.limit_angle))
 
    def write_morph(self, morphs):
        self.write_int(len(morphs), 4)
        for m in morphs:
            self.write_text(m.name)
            self.write_text(m.english_name)
            self.write_int(m.panel, 1)
            self.write_int(m.morph_type, 1)
            if m.morph_type==0:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented GroupMorph")
            elif m.morph_type==1:
                self.write_int(len(m.offsets), 4)
                for o in m.offsets:
                    self.write_vertex_index(o.vertex_index)
                    self.write_vector3(o.position_offset)
            elif m.morph_type==2:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented BoneMorph")
            elif m.morph_type==3:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented UvMorph")
            elif m.morph_type==4:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented extended UvMorph1")
            elif m.morph_type==5:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented extended UvMorph2")
            elif m.morph_type==6:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented extended UvMorph3")
            elif m.morph_type==7:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented extended UvMorph4")
            elif m.morph_type==8:
                # todo
                raise pymeshio.common.WriteException(
                        "not implemented extended MaterialMorph")
            else:
                raise pymeshio.common.WriteException(
                        "unknown morph type: {0}".format(m.morph_type))

    def write_display_slots(self, display_slots):
        self.write_int(len(display_slots), 4)
        for s in display_slots:
            self.write_text(s.name)
            self.write_text(s.english_name)
            self.write_int(s.special_flag, 1)
            self.write_int(len(s.references), 4)
            for r in s.references:
                self.write_int(r[0], 1)
                if r[0]==0:
                    self.write_bone_index(r[1])
                elif r[0]==1:
                    self.write_morph_index(r[1])
                else:
                    raise pymeshio.common.WriteException(
                            "unknown display_type: {0}".format(r[0]))

    def write_rigidbodies(self, rigidbodies):
        self.write_int(len(rigidbodies), 4)
        for rb in rigidbodies:
            self.write_text(rb.name)
            self.write_text(rb.english_name)
            self.write_bone_index(rb.bone_index)
            self.write_int(rb.collision_group, 1)
            self.write_int(rb.no_collision_group, 2)
            self.write_int(rb.shape_type, 1)
            self.write_vector3(rb.shape_size)
            self.write_vector3(rb.shape_position)
            self.write_vector3(rb.shape_rotation)
            self.write_float(rb.param.mass)
            self.write_float(rb.param.linear_damping)
            self.write_float(rb.param.angular_damping)
            self.write_float(rb.param.restitution)
            self.write_float(rb.param.friction)
            self.write_int(rb.mode, 1)

    def write_joints(self, joints):
        self.write_int(len(joints), 4)
        for j in joints:
            self.write_text(j.name)
            self.write_text(j.english_name)
            self.write_int(j.joint_type, 1)
            self.write_rigidbody_index(j.rigidbody_index_a)
            self.write_rigidbody_index(j.rigidbody_index_b)
            self.write_vector3(j.position)
            self.write_vector3(j.rotation)
            self.write_vector3(j.translation_limit_min)
            self.write_vector3(j.translation_limit_max)
            self.write_vector3(j.rotation_limit_min)
            self.write_vector3(j.rotation_limit_max)
            self.write_vector3(j.spring_constant_translation)
            self.write_vector3(j.spring_constant_rotation)


def write(ios, model, text_encoding=1):
    """
    write model to ios.

    :Parameters:
        ios
            output stream (in io.IOBase)
        model
            pmx model
        text_encoding
            text field encoding (0: UTF16, 1:UTF-8).
            0: UTF16 has bug. it write BOM(FFFE).

    >>> import pymeshio.pmx.writer
    >>> pymeshio.pmx.writer.write(io.open('out.pmx', 'wb'), pmx_model)

    """
    assert(isinstance(ios, io.IOBase))
    assert(isinstance(model, pmx.Model))
    writer=common.BinaryWriter(ios)
    # header
    writer.write_bytes(b"PMX ")
    writer.write_float(model.version)

    # flags
    writer.write_int(8, 1)
    # textencoding
    writer.write_int(text_encoding, 1)
    # extend uv
    writer.write_int(0, 1)
    def get_array_size(size):
        if size<128:
            return 1
        elif size<32768:
            return 2
        elif size<2147483647:
            return 4
        else:
            raise common.WriteError(
                    "invalid array_size: {0}".format(size))
    # vertex_index_size
    vertex_index_size=get_array_size(len(model.vertices))
    writer.write_int(vertex_index_size, 1)
    # texture_index_size
    texture_index_size=get_array_size(len(model.textures))
    writer.write_int(texture_index_size, 1)
    # material_index_size
    material_index_size=get_array_size(len(model.materials))
    writer.write_int(material_index_size, 1)
    # bone_index_size
    bone_index_size=get_array_size(len(model.bones))
    writer.write_int(bone_index_size, 1)
    # morph_index_size
    morph_index_size=get_array_size(len(model.morphs))
    writer.write_int(morph_index_size, 1)
    # rigidbody_index_size
    rigidbody_index_size=get_array_size(len(model.rigidbodies))
    writer.write_int(rigidbody_index_size, 1)

    writer=Writer(writer.ios, 
            text_encoding, 0,
            vertex_index_size, texture_index_size, material_index_size,
            bone_index_size, morph_index_size, rigidbody_index_size)
 
    # model info
    writer.write_text(model.name)
    writer.write_text(model.english_name)
    writer.write_text(model.comment)
    writer.write_text(model.english_comment)

    # model data
    writer.write_vertices(model.vertices)
    writer.write_indices(model.indices)
    writer.write_textures(model.textures)
    writer.write_materials(model.materials)
    writer.write_bones(model.bones)
    writer.write_morph(model.morphs)
    writer.write_display_slots(model.display_slots)
    writer.write_rigidbodies(model.rigidbodies)
    writer.write_joints(model.joints)
    return True

